# -*- coding: utf-8 -*-
"""test fragments feature"""

from django.conf import settings

from django.contrib.auth.models import User, Permission
from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import reverse
from django.template import Template, Context

from model_mommy import mommy
from coop_cms.models import BaseArticle, Fragment, FragmentType, FragmentFilter
from coop_cms.settings import get_article_class
from coop_cms.tests import BaseTestCase, BeautifulSoup


class BaseFragmentTest(BaseTestCase):
    """base class for fragments test"""

    def __init__(self, *args, **kwargs):
        super(BaseFragmentTest, self).__init__(*args, **kwargs)
        self.user = None

    def setUp(self):
        """before each test"""
        super(BaseFragmentTest, self).setUp()
        self._default_article_templates = settings.COOP_CMS_ARTICLE_TEMPLATES
        settings.COOP_CMS_ARTICLE_TEMPLATES = (
            ('test/article_with_fragments.html', 'Article with fragments'),
            ('test/article_with_fragments_extra_id.html', 'Article with fragments extra id'),
            ('test/article_with_fragments_template.html', 'Article with fragments template'),
        )

    def tearDown(self):
        """after each test"""
        super(BaseFragmentTest, self).tearDown()
        #restore
        settings.COOP_CMS_ARTICLE_TEMPLATES = self._default_article_templates

    def _log_as_editor(self):
        """_log as editor"""
        self.user = user = User.objects.create_user('toto', 'toto@toto.fr', 'toto')

        content_type1 = ContentType.objects.get_for_model(get_article_class())
        content_type2 = ContentType.objects.get_for_model(Fragment)

        for content_type in (content_type1, content_type2):

            perm = 'change_{0}'.format(content_type.model)
            can_edit = Permission.objects.get(content_type=content_type, codename=perm)
            user.user_permissions.add(can_edit)

            perm = 'add_{0}'.format(content_type.model)
            can_add = Permission.objects.get(content_type=content_type, codename=perm)
            user.user_permissions.add(can_add)

        user.is_active = True
        user.save()
        return self.client.login(username='toto', password='toto')

    def _log_as_regular_user(self):
        """log a reguar user"""
        user = User.objects.create_user('titi', 'titi@toto.fr', 'titi')

        #ContentType.objects.get_for_model(get_article_class())

        user.is_active = True
        user.save()
        return self.client.login(username='titi', password='titi')


class FragmentsTest(BaseFragmentTest):
    """Test ragments"""

    editable_field_tpl = '<div class="inline-editable" id="html_editor_html_editor__coop_cms__Fragment__id__{0}__content">' + \
        '{1}</div>\n<input type="hidden" id="html_editor_html_editor__coop_cms__Fragment__id__{0}__content_hidden" ' + \
        'name="html_editor__coop_cms__Fragment__id__{0}__content" value="{1}" />'
    

    def test_fragment_position(self):
        """test position is taken into account"""

        fragment_type1 = mommy.make(FragmentType)
        fragment_type2 = mommy.make(FragmentType)
        
        fragment1 = mommy.make(Fragment, type=fragment_type1)
        fragment2 = mommy.make(Fragment, type=fragment_type1)
        fragment3 = mommy.make(Fragment, type=fragment_type1)
        fragment4 = mommy.make(Fragment, type=fragment_type1)
        
        fragment_g1 = mommy.make(Fragment, type=fragment_type2)
        fragment_g2 = mommy.make(Fragment, type=fragment_type2)
        fragment_g3 = mommy.make(Fragment, type=fragment_type2)
        
        fragment5 = mommy.make(Fragment, type=fragment_type1)
        
        for idx, elt in enumerate([fragment1, fragment2, fragment3, fragment4, fragment5]):
            self.assertEqual(idx+1, elt.position)
        
        for idx, elt in enumerate([fragment_g1, fragment_g2, fragment_g3]):
            self.assertEqual(idx+1, elt.position)
            
    def test_fragment_position_extra_id(self):
        """test position is taken into account when extra id is defined"""

        fragment_type1 = mommy.make(FragmentType)
        fragment_type2 = mommy.make(FragmentType)
        fragment_filter1 = mommy.make(FragmentFilter)
        fragment_filter2 = mommy.make(FragmentFilter)
        
        fragments_1 = [
            mommy.make(Fragment, type=fragment_type1, filter=fragment_filter1),
            mommy.make(Fragment, type=fragment_type1, filter=fragment_filter1),
            mommy.make(Fragment, type=fragment_type1, filter=fragment_filter1),
            mommy.make(Fragment, type=fragment_type1, filter=fragment_filter2),
            mommy.make(Fragment, type=fragment_type1, filter=fragment_filter1),
            mommy.make(Fragment, type=fragment_type1),
            mommy.make(Fragment, type=fragment_type1),
        ]

        fragments_2 = [
            mommy.make(Fragment, type=fragment_type2, filter=fragment_filter1),
            mommy.make(Fragment, type=fragment_type2, filter=fragment_filter2),
            mommy.make(Fragment, type=fragment_type2, filter=fragment_filter2),
        ]

        for idx, elt in enumerate([fragments_1[0], fragments_1[1], fragments_1[2], fragments_1[4]]):
            self.assertEqual(idx+1, elt.position)
            
        for idx, elt in enumerate([fragments_1[3]]):
            self.assertEqual(idx+1, elt.position)
        
        for idx, elt in enumerate([fragments_2[0]]):
            self.assertEqual(idx+1, elt.position)
            
        for idx, elt in enumerate([fragments_2[1], fragments_2[2]]):
            self.assertEqual(idx+1, elt.position)
            
        for idx, elt in enumerate([fragments_1[5], fragments_1[6]]):
            self.assertEqual(idx+1, elt.position)
            
    def test_fragment_position_update(self):
        """test position can be modified"""

        fragment_type1 = mommy.make(FragmentType)
        mommy.make(FragmentType)
        
        fragment1 = mommy.make(Fragment, type=fragment_type1)
        fragment2 = mommy.make(Fragment, type=fragment_type1)
        fragment3 = mommy.make(Fragment, type=fragment_type1)
        
        fragment1.save()
        fragment2.save()
        fragment3.save()
        
        for idx, elt in enumerate([fragment1, fragment2, fragment3]):
            self.assertEqual(idx+1, elt.position)
            
    def test_view_fragments(self):
        """test view fragments"""

        ft_name = u"contacts"
        fragment_type1 = mommy.make(FragmentType, name=ft_name)
        
        fragment1 = mommy.make(Fragment, type=fragment_type1, content="Azerty")
        fragment2 = mommy.make(Fragment, type=fragment_type1, content="Qsdfgh")
        fragment3 = mommy.make(Fragment, type=fragment_type1, content="Wxcvbn")
        
        tpl = Template('{% load coop_edition %}{% coop_fragments ft_name %}')
        html = tpl.render(Context({"ft_name": ft_name}))
        
        positions = [html.find('{0}'.format(f.content)) for f in [fragment1, fragment2, fragment3]]
        for pos in positions:
            self.assertTrue(pos >= 0)
        sorted_positions = positions[:]
        sorted_positions.sort()
        self.assertEqual(positions, sorted_positions)
        
    def test_view_fragments_extra_id_in_edit_mode(self):
        """in edit mode: coop-fragment-type"""
        ft_name = u"contacts"
        fragment_type1 = mommy.make(FragmentType, name=ft_name)
        fragment_filter1 = mommy.make(FragmentFilter, extra_id="1")
        fragment_filter2 = mommy.make(FragmentFilter, extra_id="2")
        
        fragments = [
            mommy.make(Fragment, type=fragment_type1, content="Azerty", filter=fragment_filter1),
            mommy.make(Fragment, type=fragment_type1, content="Qsdfgh", filter=fragment_filter1),
            mommy.make(Fragment, type=fragment_type1, content="Wxcvbn", filter=fragment_filter2),
            mommy.make(Fragment, type=fragment_type1, content="Zsxdrg", filter=None),
        ]
        
        tpl = Template('{% load coop_edition %}{% coop_fragments ft_name x %}')
        html = tpl.render(Context({"ft_name": ft_name, "x": 1, "form": "dummy_form"}))
        
        positions = [html.find('{0}'.format(f.content)) for f in [fragments[0], fragments[1]]]
        for pos in positions:
            self.assertTrue(pos >= 0)
        self.assertEqual(positions, sorted(positions))
        
        soup = BeautifulSoup(html)
        ft_tags = soup.select(".coop-fragment-type")
        self.assertEqual(len(ft_tags), 1)
        ft_tag = ft_tags[0]
        self.assertEqual(ft_tag['rel'], str(fragment_type1.id))
        self.assertEqual(ft_tag['data-filter'], str(fragment_filter1.id))

        for frag in [fragments[2], fragments[3]]:
            self.assertTrue(html.find(frag.content) < 0)

    def test_view_fragments_extra_id_in_view_mode(self):
        """in view mode: no coop-fragment-type"""
        ft_name = u"contacts"
        fragment_type1 = mommy.make(FragmentType, name=ft_name)
        fragment_filter1 = mommy.make(FragmentFilter, extra_id="1")
        fragment_filter2 = mommy.make(FragmentFilter, extra_id="2")

        fragments = [
            mommy.make(Fragment, type=fragment_type1, content="Azerty", filter=fragment_filter1),
            mommy.make(Fragment, type=fragment_type1, content="Qsdfgh", filter=fragment_filter1),
            mommy.make(Fragment, type=fragment_type1, content="Wxcvbn", filter=fragment_filter2),
            mommy.make(Fragment, type=fragment_type1, content="Zsxdrg", filter=None),
        ]

        tpl = Template('{% load coop_edition %}{% coop_fragments ft_name x %}')
        html = tpl.render(Context({"ft_name": ft_name, "x": 1}))

        positions = [html.find('{0}'.format(f.content)) for f in [fragments[0], fragments[1]]]
        for pos in positions:
            self.assertTrue(pos >= 0)
        self.assertEqual(positions, sorted(positions))

        soup = BeautifulSoup(html)
        ft_tags = soup.select(".coop-fragment-type")
        self.assertEqual(len(ft_tags), 0)

        for frag in [fragments[2], fragments[3]]:
            self.assertTrue(html.find(frag.content) < 0)

    def test_fragments_with_extra_id(self):
        """test fragments with extra id"""

        ft_name = u"contacts"
        
        tpl = Template('{% load coop_edition %}{% coop_fragments ft_name x %}')
        tpl.render(Context({"ft_name": ft_name, 'x': 2}))
        
        self.assertEqual(FragmentType.objects.count(), 1)
        self.assertEqual(FragmentType.objects.filter(name=ft_name).count(), 1)

        self.assertEqual(FragmentFilter.objects.count(), 1)
        self.assertEqual(FragmentFilter.objects.filter(extra_id='2').count(), 1)
        
    def test_view_fragments_name_as_string(self):
        """test fragments name hardcode in templatetag"""

        fragment_type1 = mommy.make(FragmentType, name="contacts")
        
        fragment1 = mommy.make(Fragment, type=fragment_type1, content="Azerty")
        fragment2 = mommy.make(Fragment, type=fragment_type1, content="Qsdfgh")
        fragment3 = mommy.make(Fragment, type=fragment_type1, content="Wxcvbn")
        
        tpl = Template('{% load coop_edition %}{% coop_fragments "contacts" %}')
        html = tpl.render(Context({"ft_name": "contacts"}))
        
        positions = [html.find('{0}'.format(f.content)) for f in [fragment1, fragment2, fragment3]]
        for pos in positions:
            self.assertTrue(pos >= 0)
        sorted_positions = positions[:]
        sorted_positions.sort()
        self.assertEqual(positions, sorted_positions)
        
    def test_view_fragments_args_as_string(self):
        """test name and extra id hardcoded"""

        fragment_type1 = mommy.make(FragmentType, name="contacts")
        fragment_filter1 = mommy.make(FragmentFilter, extra_id="hello")
        fragment_filter2 = mommy.make(FragmentFilter, extra_id="2")
        
        fragment1 = mommy.make(Fragment, type=fragment_type1, content="Azerty", filter=fragment_filter1)
        fragment2 = mommy.make(Fragment, type=fragment_type1, content="Qsdfgh", filter=fragment_filter1)
        fragment3 = mommy.make(Fragment, type=fragment_type1, content="Wxcvbn", filter=fragment_filter2)
        fragment4 = mommy.make(Fragment, type=fragment_type1, content="Zsxdrg", filter=None)
        
        tpl = Template('{% load coop_edition %}{% coop_fragments "contacts" "hello" %}')
        html = tpl.render(Context({"ft_name": "contacts"}))
        
        positions = [html.find('{0}'.format(f.content)) for f in [fragment1, fragment2]]
        for pos in positions:
            self.assertTrue(pos >= 0)
        sorted_positions = positions[:]
        sorted_positions.sort()
        self.assertEqual(positions, sorted_positions)
        
        for frag in [fragment3, fragment4]:
            self.assertTrue(html.find(frag.content) < 0)
        
    def test_view_fragments_order(self):
        """test fragments displayed in position order"""

        ft_name = u"contacts"
        fragment_type1 = mommy.make(FragmentType, name=ft_name)
        
        fragment1 = mommy.make(Fragment, type=fragment_type1, content="Azerty", position=3)
        fragment2 = mommy.make(Fragment, type=fragment_type1, content="Qsdfgh", position=1)
        fragment3 = mommy.make(Fragment, type=fragment_type1, content="Wxcvbn", position=2)
        
        tpl = Template('{% load coop_edition %}{% coop_fragments ft_name %}')
        html = tpl.render(Context({"ft_name": ft_name}))
        
        positions = [html.find('{0}'.format(f.content)) for f in [fragment2, fragment3, fragment1]]
        for pos in positions:
            self.assertTrue(pos >= 0)
        sorted_positions = positions[:]
        sorted_positions.sort()
        self.assertEqual(positions, sorted_positions)
        
    def test_view_only_specified_fragments(self):
        """test display only right fragements"""

        ft_name = u"contacts"
        fragment_type1 = mommy.make(FragmentType, name=ft_name)
        fragment_type2 = mommy.make(FragmentType, name="AAAA")
        
        fragment1 = mommy.make(Fragment, type=fragment_type1, content="Azerty")
        fragment2 = mommy.make(Fragment, type=fragment_type1, content="Qsdfgh")
        fragment3 = mommy.make(Fragment, type=fragment_type1, content="Wxcvbn")
        
        fragment_g1 = mommy.make(Fragment, type=fragment_type2, content="POIUYT")
        
        tpl = Template('{% load coop_edition %}{% coop_fragments ft_name %}')
        html = tpl.render(Context({"ft_name": ft_name}))
        
        positions = [html.find('{0}'.format(f.content)) for f in [fragment2, fragment3, fragment1]]
        for pos in positions:
            self.assertTrue(pos >= 0)
        
        positions = [html.find('{0}'.format(f.content)) for f in [fragment_g1]]
        for pos in positions:
            self.assertTrue(pos == -1)
            
    def test_view_only_specified_extra_id(self):
        """text extra_id is taken into account"""
        ft_name = u"contacts"
        fragment_type1 = mommy.make(FragmentType, name=ft_name)
        fragment_type2 = mommy.make(FragmentType, name="AAAA")
        
        fragment_filter1 = mommy.make(FragmentFilter, extra_id="hello")
        fragment_filter2 = mommy.make(FragmentFilter, extra_id="2")
        
        fragments = [
            mommy.make(Fragment, type=fragment_type1, content="Azerty", filter=fragment_filter1),
            mommy.make(Fragment, type=fragment_type1, content="Qsdfgh", filter=fragment_filter1),
            mommy.make(Fragment, type=fragment_type1, content="Wxcvbn", filter=fragment_filter2),
            mommy.make(Fragment, type=fragment_type1, content="Zsxdrg", filter=None),
            mommy.make(Fragment, type=fragment_type2, content="POIUYT", filter=fragment_filter1),
        ]
        
        tpl = Template('{% load coop_edition %}{% coop_fragments ft_name "hello" %}')
        html = tpl.render(Context({"ft_name": ft_name}))
        
        positions = [html.find('{0}'.format(f.content)) for f in [fragments[1], fragments[0]]]
        for pos in positions:
            self.assertTrue(pos >= 0)
        
        positions = [html.find('{0}'.format(f.content)) for f in [fragments[4], fragments[2], fragments[3]]]
        for pos in positions:
            self.assertTrue(pos == -1)

    def test_view_extra_id_named_args(self):
        """text extra_id is taken into account extra_id is given as named arg"""
        ft_name = u"contacts"
        fragment_type1 = mommy.make(FragmentType, name=ft_name)
        fragment_type2 = mommy.make(FragmentType, name="AAAA")

        fragment_filter1 = mommy.make(FragmentFilter, extra_id="hello")
        fragment_filter2 = mommy.make(FragmentFilter, extra_id="2")

        fragments = [
            mommy.make(Fragment, type=fragment_type1, content="Azerty", filter=fragment_filter1),
            mommy.make(Fragment, type=fragment_type1, content="Qsdfgh", filter=fragment_filter1),
            mommy.make(Fragment, type=fragment_type1, content="Wxcvbn", filter=fragment_filter2),
            mommy.make(Fragment, type=fragment_type1, content="Zsxdrg", filter=None),
            mommy.make(Fragment, type=fragment_type2, content="POIUYT", filter=fragment_filter1),
        ]

        tpl = Template('{% load coop_edition %}{% coop_fragments ft_name extra_id="hello" %}')
        html = tpl.render(Context({"ft_name": ft_name}))

        positions = [html.find('{0}'.format(f.content)) for f in [fragments[1], fragments[0]]]
        for pos in positions:
            self.assertTrue(pos >= 0)

        positions = [html.find('{0}'.format(f.content)) for f in [fragments[4], fragments[2], fragments[3]]]
        for pos in positions:
            self.assertTrue(pos == -1)

    def test_view_extra_id_named_args_end(self):
        """text extra_id is taken into account extra_id is given as named arg in last position"""
        ft_name = u"contacts"
        fragment_type1 = mommy.make(FragmentType, name=ft_name)
        fragment_type2 = mommy.make(FragmentType, name="AAAA")

        fragment_filter1 = mommy.make(FragmentFilter, extra_id="hello")
        fragment_filter2 = mommy.make(FragmentFilter, extra_id="2")

        fragments = [
            mommy.make(Fragment, type=fragment_type1, content="Azerty", filter=fragment_filter1),
            mommy.make(Fragment, type=fragment_type1, content="Qsdfgh", filter=fragment_filter1),
            mommy.make(Fragment, type=fragment_type1, content="Wxcvbn", filter=fragment_filter2),
            mommy.make(Fragment, type=fragment_type1, content="Zsxdrg", filter=None),
            mommy.make(Fragment, type=fragment_type2, content="POIUYT", filter=fragment_filter1),
        ]

        tpl = Template(
            '{% load coop_edition %}{% coop_fragments ft_name template_name="test/_fragment.html" extra_id="hello" %}'
        )
        html = tpl.render(Context({"ft_name": ft_name}))

        positions = [html.find('{0}'.format(f.content)) for f in [fragments[1], fragments[0]]]
        for pos in positions:
            self.assertTrue(pos >= 0)

        positions = [html.find('{0}'.format(f.content)) for f in [fragments[4], fragments[2], fragments[3]]]
        for pos in positions:
            self.assertTrue(pos == -1)
            
    def test_view_fragments_edit_mode(self):
        """test view in edit mode"""

        ft_name = u"contacts"
        fragment_type1 = mommy.make(FragmentType, name=ft_name)
        fragment_type2 = mommy.make(FragmentType, name="AAAA")
        
        fragments = [
            mommy.make(Fragment, type=fragment_type1, content="Azerty"),
            mommy.make(Fragment, type=fragment_type1, content="Qsdfgh"),
            mommy.make(Fragment, type=fragment_type1, content="Wxcvbn"),
            mommy.make(Fragment, type=fragment_type2, content="POIUYT"),
        ]

        tpl = Template('{% load coop_edition %}{% coop_fragments ft_name %}')
        html = tpl.render(Context({"ft_name": ft_name, "form": True}))
        
        positions = [
            html.find(self.editable_field_tpl.format(f.id, f.content))
            for f in [fragments[0], fragments[1], fragments[2]]
        ]
        for pos in positions:
            self.assertTrue(pos >= 0)
        self.assertEqual(positions, sorted(positions))
        
        positions = [html.find(self.editable_field_tpl.format(f.id, f.content)) for f in [fragments[3]]]
        for pos in positions:
            self.assertTrue(pos == -1)
            
    def test_view_fragments_extra_id_edit_mode(self):
        """test view with extra id in edit mode"""
        ft_name = u"contacts"
        fragment_type1 = mommy.make(FragmentType, name=ft_name)
        fragment_type2 = mommy.make(FragmentType, name="AAAA")
        
        fragment_filter1 = mommy.make(FragmentFilter, extra_id="hello")
        fragment_filter2 = mommy.make(FragmentFilter, extra_id="2")
        
        fragments = [
            mommy.make(Fragment, type=fragment_type1, content="Azerty", filter=fragment_filter1),
            mommy.make(Fragment, type=fragment_type1, content="Qsdfgh", filter=fragment_filter1),
            mommy.make(Fragment, type=fragment_type1, content="Wxcvbn", filter=fragment_filter2),
            mommy.make(Fragment, type=fragment_type1, content="Zsxdrg", filter=None),
            mommy.make(Fragment, type=fragment_type2, content="POIUYT")
        ]
        
        tpl = Template('{% load coop_edition %}{% coop_fragments ft_name "hello" %}')
        html = tpl.render(Context({"ft_name": ft_name, "form": True}))

        positions = [
            html.find(self.editable_field_tpl.format(fragment.id, fragment.content))
            for fragment in [fragments[0], fragments[1]]
        ]
        for pos in positions:
            self.assertTrue(pos >= 0)
        self.assertEqual(positions, sorted(positions))
        
        positions = [
            html.find(self.editable_field_tpl.format(fragment.id, fragment.content))
            for fragment in [fragments[4], fragments[2], fragments[3]]
        ]
        for pos in positions:
            self.assertTrue(pos == -1)
            
    def test_fragments_with_template(self):
        """test template_name argument of the template tag"""
        ft_name = u"contacts"
        
        tpl = Template('{% load coop_edition %}{% coop_fragments ft_name template_name="test/_fragment.html" %}')
        html = tpl.render(Context({"ft_name": ft_name}))
        
        self.assertEqual(FragmentType.objects.count(), 1)
        self.assertEqual(FragmentType.objects.filter(name=ft_name).count(), 1)
        
        soup = BeautifulSoup(html)
        self.assertEqual(0, len(soup.select('.panel')))
        
    def test_view_fragments_with_template(self):
        """test view with template_name arguement"""
        ft_name = u"contacts"
        fragment_type = mommy.make(FragmentType, name=ft_name)
        
        mommy.make(Fragment, type=fragment_type)
        
        tpl = Template('{% load coop_edition %}{% coop_fragments ft_name template_name="test/_fragment.html" %}')
        html = tpl.render(Context({"ft_name": ft_name}))
        
        self.assertEqual(FragmentType.objects.count(), 1)
        self.assertEqual(FragmentType.objects.filter(name=ft_name).count(), 1)
        
        soup = BeautifulSoup(html)
        self.assertEqual(1, len(soup.select('.panel')))
        
    def test_view_fragments_template_edit_mode(self):
        """test with template_name in edit_mode"""
        ft_name = u"contacts"
        fragment_type = mommy.make(FragmentType, name=ft_name)
        
        mommy.make(Fragment, type=fragment_type)
        
        tpl = Template('{% load coop_edition %}{% coop_fragments ft_name template_name="test/_fragment.html" %}')
        html = tpl.render(Context({"ft_name": ft_name, 'form': True}))
        
        self.assertEqual(FragmentType.objects.count(), 1)
        self.assertEqual(FragmentType.objects.filter(name=ft_name).count(), 1)
        
        soup = BeautifulSoup(html)
        self.assertEqual(1, len(soup.select('.panel')))
        self.assertEqual(1, len(soup.select('.panel input')))
        self.assertEqual(1, len(soup.select('.panel .inline-editable')))
    
    def test_view_fragments_with_template2(self):
        """test with another template"""
        ft_name = u"contacts"
        fragment_type = mommy.make(FragmentType, name=ft_name)
        
        mommy.make(Fragment, type=fragment_type)
        mommy.make(Fragment, type=fragment_type)
        
        tpl = Template('{% load coop_edition %}{% coop_fragments ft_name template_name="test/_fragment.html" %}')
        html = tpl.render(Context({"ft_name": ft_name}))
        
        self.assertEqual(FragmentType.objects.count(), 1)
        self.assertEqual(FragmentType.objects.filter(name=ft_name).count(), 1)
        soup = BeautifulSoup(html)
        self.assertEqual(2, len(soup.select('.panel')))
        
    def test_view_fragments_with_template3(self):
        """test with another other template"""
        ft_name = u"contacts"
        fragment_type = mommy.make(FragmentType, name=ft_name)
        
        mommy.make(Fragment, type=fragment_type)
        mommy.make(Fragment, type=fragment_type)
        
        tpl = Template('{% load coop_edition %}{% coop_fragments ft_name template_name="test/_fragment.html" %}')
        html = tpl.render(Context({"ft_name": ft_name, 'form': True}))
        
        self.assertEqual(FragmentType.objects.count(), 1)
        self.assertEqual(FragmentType.objects.filter(name=ft_name).count(), 1)
        soup = BeautifulSoup(html)
        self.assertEqual(3, len(soup.select('.panel'))) # 1 extra panel if_cms_edition and fragment index > 0


class FragmentsInArticleTest(BaseFragmentTest):
    """Articles related tests"""

    def _check_article(self, response, data):
        """check page content"""
        for value in data.values():
            self.assertContains(response, value)

    def test_view_article_no_fragments(self):
        """view article with no Fragment"""
        template = settings.COOP_CMS_ARTICLE_TEMPLATES[0][0]
        article = get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)
        response = self.client.get(article.get_absolute_url())
        self.assertEqual(200, response.status_code)
        self.assertEqual(1, FragmentType.objects.count())
        self.assertEqual("parts", FragmentType.objects.all()[0].name)
        
    def test_view_article_with_fragments(self):
        """view article with Fragment"""
        template = settings.COOP_CMS_ARTICLE_TEMPLATES[0][0]
        
        article = get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)
        
        fragment_type = mommy.make(FragmentType, name="parts")
        fragment1 = mommy.make(Fragment, type=fragment_type, content="Azertyuiop")
        
        response = self.client.get(article.get_absolute_url())
        
        self.assertEqual(200, response.status_code)
        self.assertContains(response, fragment1.content)
        
    def test_view_article_with_fragments_extra_id(self):
        """view article with Fragment and extra_id"""
        template = settings.COOP_CMS_ARTICLE_TEMPLATES[1][0]
        article = get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)
        
        fragment_type = mommy.make(FragmentType, name="parts")
        fragment_filter1 = mommy.make(FragmentFilter, extra_id=str(article.id))
        fragment_filter2 = mommy.make(FragmentFilter, extra_id="hello")
        fragment1 = mommy.make(Fragment, type=fragment_type, content="Azertyuiop", filter=fragment_filter1)
        fragment2 = mommy.make(Fragment, type=fragment_type, content="QSDFGHJKLM", filter=fragment_filter2)
        fragment3 = mommy.make(Fragment, type=fragment_type, content="Wxcvbn,;:=", filter=None)
        
        response = self.client.get(article.get_absolute_url())
        self.assertEqual(200, response.status_code)
        self.assertContains(response, fragment1.content)
        self.assertNotContains(response, fragment2.content)
        self.assertNotContains(response, fragment3.content)
        
    def test_view_article_with_fragment_with_css(self):
        """view article with Fragment and css"""

        template = settings.COOP_CMS_ARTICLE_TEMPLATES[0][0]
        article = get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)
        
        fragment_type = mommy.make(FragmentType, name="parts")
        fragment1 = mommy.make(Fragment, type=fragment_type, content="Azertyuiop", css_class="this-is-my-fragment")
        
        response = self.client.get(article.get_absolute_url())
        self.assertEqual(200, response.status_code)
        self.assertContains(response, fragment1.content)
        
        soup = BeautifulSoup(response.content)
        fragment = soup.select("div."+fragment1.css_class)[0]
        self.assertEqual(fragment1.content, fragment.text)
        
    def test_edit_article_no_fragments(self):
        """edit article with no Fragment"""

        template = settings.COOP_CMS_ARTICLE_TEMPLATES[0][0]
        article = get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)
        
        data = {"title": 'salut', 'content': 'bonjour!'}
        
        self._log_as_editor()
        response = self.client.post(article.get_edit_url(), data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        self._check_article(response, data)
        
        data = {"title": 'bye', 'content': 'au revoir'}
        response = self.client.post(article.get_edit_url(), data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        self._check_article(response, data)
        
    def test_edit_article_with_fragments(self):
        """edit article with Fragment"""

        template = settings.COOP_CMS_ARTICLE_TEMPLATES[0][0]
        article = get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)
        
        fragment_type = mommy.make(FragmentType, name="parts")
        fragment1 = mommy.make(Fragment, type=fragment_type, content="Azertyuiop")
        
        new_fragment1_content = u"Qsdfghjklm"
        data = {
            "title": 'salut',
            'content': 'bonjour!',
            'html_editor__coop_cms__Fragment__id__{0}__content'.format(fragment1.id): new_fragment1_content,
        }
        
        self._log_as_editor()
        response = self.client.post(article.get_edit_url(), data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, data['title'])
        self.assertContains(response, data['content'])
        self.assertContains(response, new_fragment1_content)
        
    def test_edit_article_with_fragments_extra_id(self):
        """edit article with Fragment and extra_id"""

        template = settings.COOP_CMS_ARTICLE_TEMPLATES[1][0]
        article = get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)
        
        fragment_type = mommy.make(FragmentType, name="parts")
        fragment_filter = mommy.make(FragmentFilter, extra_id=str(article.id))
        fragment1 = mommy.make(Fragment, type=fragment_type, content="Azertyuiop", filter=fragment_filter)
        
        new_fragment1_content = u"Qsdfghjklm"
        data = {
            "title": 'salut',
            'content': 'bonjour!',
            'html_editor__coop_cms__Fragment__id__{0}__content'.format(fragment1.id): new_fragment1_content,
        }
        
        self._log_as_editor()
        response = self.client.post(article.get_edit_url(), data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, data['title'])
        self.assertContains(response, data['content'])
        self.assertContains(response, new_fragment1_content)
        
    def test_view_add_fragment(self):
        """can view add fragment"""
        template = settings.COOP_CMS_ARTICLE_TEMPLATES[0][0]
        get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)
        
        self._log_as_editor()
        
        url = reverse("coop_cms_add_fragment")
        response = self.client.get(url)
        
        self.assertEqual(200, response.status_code)
        
    def test_view_add_fragment_check_filters(self):
        """add fragment check filters"""
        template = settings.COOP_CMS_ARTICLE_TEMPLATES[1][0]
        article = get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)
        
        self._log_as_editor()
        
        url = article.get_edit_url()
        response = self.client.get(url)
        
        self.assertEqual(200, response.status_code)
        soup = BeautifulSoup(response.content)
        
        ft_tags = soup.select(".coop-fragment-type")
        ft_objs = FragmentType.objects.all()
        ff_objs = FragmentFilter.objects.all()
        
        self.assertEqual(len(ft_tags), 2)
        self.assertEqual(ft_objs.count(), 2)
        self.assertEqual(ff_objs.count(), 1)
        
        for i in range(2):
            self.assertEqual(int(ft_tags[i]["rel"]), ft_objs[i].id)
        
        self.assertEqual(ft_tags[0]["data-filter"], '')
        self.assertEqual(ft_tags[1]["data-filter"], str(ff_objs[0].id))
        
    def test_view_add_fragment_no_filter_check(self):
        """view add fragment"""
        template = settings.COOP_CMS_ARTICLE_TEMPLATES[0][0]
        article = get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)
        
        self._log_as_editor()
        
        url = article.get_edit_url()
        response = self.client.get(url)
        
        self.assertEqual(200, response.status_code)
        soup = BeautifulSoup(response.content)
        
        ft_tags = soup.select(".coop-fragment-type")
        ft_objs = FragmentType.objects.all()
        ff_objs = FragmentFilter.objects.all()
        
        self.assertEqual(len(ft_tags), 1)
        self.assertEqual(ft_objs.count(), 1)
        self.assertEqual(ff_objs.count(), 0)
        
        self.assertEqual(int(ft_tags[0]["rel"]), ft_objs[0].id)
        
        self.assertEqual(ft_tags[0]["data-filter"], '')
        
    def test_view_add_fragment_permission_denied(self):
        """view add fragment not allowed"""
        template = settings.COOP_CMS_ARTICLE_TEMPLATES[0][0]
        get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)
        
        url = reverse("coop_cms_add_fragment")
        response = self.client.get(url)
        self.assertEqual(302, response.status_code)
        
        self._log_as_regular_user()
        response = self.client.get(url)
        self.assertEqual(403, response.status_code)
        
    def _add_fragment(self, data, errors_count=0):
        """helper"""
        template = settings.COOP_CMS_ARTICLE_TEMPLATES[0][0]
        get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)
        
        self._log_as_editor()
        
        url = reverse("coop_cms_add_fragment")
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(200, response.status_code)
        
        soup = BeautifulSoup(response.content)
        errs = soup.select("ul.errorlist li")
        if errors_count:
            self.assertEqual(errors_count, len(errs))
        else:
            self.assertEqual([], errs)
            expected = u'<script>$.colorbox.close(); window.location=window.location;</script>'.format()
            self.assertEqual(response.content, expected)
        
        return response        
        
    def test_add_fragment(self):
        """add fragment"""
        fragment_type = mommy.make(FragmentType, name="parts")
        data = {
            'type': fragment_type.id,
            'name': 'abcd',
            'position': 0,
            'filter': '',
        }
        
        self._add_fragment(data)
        fragment = Fragment.objects.all()[0]
        
        self.assertEqual(fragment.type, fragment_type)
        self.assertEqual(fragment.name, data['name'])
        self.assertEqual(fragment.css_class, '')
        self.assertEqual(fragment.position, 1)
        self.assertEqual(fragment.filter, None)
        
    def test_add_fragment_filter(self):
        """add fragment with filter"""
        fragment_type = mommy.make(FragmentType, name="parts")
        fragment_filter = mommy.make(FragmentFilter, extra_id="2")
        data = {
            'type': fragment_type.id,
            'name': 'abcd',
            'position': 0,
            'filter': fragment_filter.id
        }
        
        self._add_fragment(data)
        fragment = Fragment.objects.all()[0]
        
        self.assertEqual(fragment.type, fragment_type)
        self.assertEqual(fragment.name, data['name'])
        self.assertEqual(fragment.css_class, '')
        self.assertEqual(fragment.position, 1)
        self.assertEqual(fragment.filter, fragment_filter)
        
    def test_add_fragment_position(self):
        """add fragment with position"""
        fragment_type = mommy.make(FragmentType, name="parts")
        data = {
            'type': fragment_type.id,
            'name': 'abcd',
            'position': 2,
            'filter': '',
        }
        
        self._add_fragment(data)
        fragment = Fragment.objects.all()[0]
        
        self.assertEqual(fragment.type, fragment_type)
        self.assertEqual(fragment.name, data['name'])
        self.assertEqual(fragment.css_class, '')
        self.assertEqual(fragment.position, 2)
        
    def test_add_fragment_invalid_filter(self):
        """add fragment invalid filter"""
        fragment_type = mommy.make(FragmentType, name="parts")
        data = {
            'type': fragment_type.id,
            'name': 'abcd',
            'position': 2,
            'filter': '0',
        }
        
        self._add_fragment(data, 1)
        self.assertEqual(0, Fragment.objects.count())
        
    def test_add_fragment_one_css(self):
        """add fragment css."""
        fragment_type = mommy.make(FragmentType, name="parts", allowed_css_classes="col-1,first-line")
        data = {
            'type': fragment_type.id,
            'name': 'abcd',
            'css_class': ['col-1'],
            'position': 0,
        }

        self._add_fragment(data)
        fragment = Fragment.objects.all()[0]

        self.assertEqual(fragment.type, fragment_type)
        self.assertEqual(fragment.name, data['name'])
        self.assertEqual(fragment.css_class, 'col-1')
        self.assertEqual(fragment.position, 1)

    def test_add_fragment_two_css(self):
        """add fragment css"""
        fragment_type = mommy.make(FragmentType, name="parts", allowed_css_classes="col-1,first-line")
        data = {
            'type': fragment_type.id,
            'name': 'abcd',
            'css_class': ['col-1', 'first-line'],
            'position': 0,
        }

        self._add_fragment(data)
        fragment = Fragment.objects.all()[0]

        self.assertEqual(fragment.type, fragment_type)
        self.assertEqual(fragment.name, data['name'])
        self.assertEqual(fragment.css_class, 'col-1 first-line')
        self.assertEqual(fragment.position, 1)

    def test_add_fragment_invalid_css(self):
        """add fragment css"""
        fragment_type = mommy.make(FragmentType, name="parts", allowed_css_classes="col-1")
        data = {
            'type': fragment_type.id,
            'name': 'abcd',
            'css_class': ['col-1', 'first-line'],
            'position': 0,
        }

        self._add_fragment(data, errors_count=1)

    def test_add_fragment_unknown_css(self):
        """add fragment css"""
        fragment_type = mommy.make(FragmentType, name="parts")
        data = {
            'type': fragment_type.id,
            'name': 'abcd',
            'css_class': 'okidki',
            'position': 0,
        }
        
        self._add_fragment(data, errors_count=1)
            
    def test_view_add_fragment_no_perm(self):
        """add fragment not allowed"""
        template = settings.COOP_CMS_ARTICLE_TEMPLATES[0][0]
        get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)
        
        fragment_type = mommy.make(FragmentType, name="parts")
        data = {
            'type': fragment_type,
            'name': 'abcd',
            'css_class': 'okidoki',
            'position': 0,
        }
        
        url = reverse("coop_cms_add_fragment")
        response = self.client.post(url, data=data, follow=False)
        self.assertEqual(302, response.status_code)
        next_url = "/accounts/login/?next={0}".format(url)
        self.assertTrue(response['Location'].find(next_url) >= 0)
        
        self._log_as_regular_user()
        response = self.client.post(url, data=data, follow=False)
        self.assertEqual(403, response.status_code)
        
        self.assertEqual(0, Fragment.objects.count())
        
    def test_view_edit_fragments_empty(self):
        """view edit fragment form: no fragments yet"""
        template = settings.COOP_CMS_ARTICLE_TEMPLATES[0][0]
        get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)
        
        self._log_as_editor()
        
        url = reverse("coop_cms_edit_fragments")
        response = self.client.get(url)
        
        self.assertEqual(200, response.status_code)
        
    def test_view_edit_fragments(self):
        """view edit fragment form"""

        template = settings.COOP_CMS_ARTICLE_TEMPLATES[0][0]
        get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)
        
        fragment1 = mommy.make(Fragment, name="azerty")
        fragment2 = mommy.make(Fragment, name="qwerty")
        
        self._log_as_editor()
        
        url = reverse("coop_cms_edit_fragments")
        response = self.client.get(url)
        
        self.assertEqual(200, response.status_code)
        self.assertContains(response, fragment1.name)
        self.assertContains(response, fragment2.name)
        
    def test_view_edit_fragments_perms(self):
        """view edit fragment form: not allowed"""

        template = settings.COOP_CMS_ARTICLE_TEMPLATES[0][0]
        get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)
        
        url = reverse("coop_cms_edit_fragments")
        response = self.client.get(url)
        
        self.assertEqual(302, response.status_code)
        
        self._log_as_regular_user()
        response = self.client.get(url)
        self.assertEqual(403, response.status_code)    
    
    def test_edit_fragment(self):
        """edit fragment"""

        template = settings.COOP_CMS_ARTICLE_TEMPLATES[0][0]
        get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)
        
        fragment_type1 = mommy.make(FragmentType)
        fragment_type2 = mommy.make(FragmentType)
        fragment1 = mommy.make(Fragment, name="azerty", type=fragment_type1)
        fragment2 = mommy.make(Fragment, name="qwerty", type=fragment_type2)
        
        data = {
            'form-0-id': fragment1.id,
            'form-0-type': fragment1.type.id,
            'form-0-name': fragment1.name+"!",
            'form-0-css_class': [],
            'form-0-position': 5,
            'form-0-delete_me': False,
            
            'form-1-id': fragment2.id,
            'form-1-type': fragment2.type.id,
            'form-1-name': fragment2.name+"+",
            'form-1-css_class': [],
            'form-1-position': 2,
            'form-1-delete_me': False,
            
            'form-TOTAL_FORMS': 2,
            'form-INITIAL_FORMS': 2,
            'form-MAX_NUM_FORMS': 2
        }
        
        self._log_as_editor()
        
        url = reverse("coop_cms_edit_fragments")
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(200, response.status_code)
        
        soup = BeautifulSoup(response.content)
        errs = soup.select("ul.errorlist li")
        self.assertEqual([], errs)
        
        expected = u'<script>$.colorbox.close(); window.location=window.location;</script>'.format()
        self.assertEqual(response.content, expected)
        
        self.assertEqual(2, Fragment.objects.count())
        fragment1 = Fragment.objects.get(id=fragment1.id)
        fragment2 = Fragment.objects.get(id=fragment2.id)

        self.assertEqual(fragment1.type, fragment_type1)
        self.assertEqual(fragment1.name, "azerty!")
        self.assertEqual(fragment1.css_class, "")
        self.assertEqual(fragment1.position, 5)

        self.assertEqual(fragment2.type, fragment_type2)
        self.assertEqual(fragment2.name, "qwerty+")
        self.assertEqual(fragment2.css_class, "")
        self.assertEqual(fragment2.position, 2)

    def test_edit_fragment_css_allowed(self):
        """edit fragment: css is allowed"""

        template = settings.COOP_CMS_ARTICLE_TEMPLATES[0][0]
        get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)

        fragment_type1 = mommy.make(FragmentType, allowed_css_classes="oups")
        fragment_type2 = mommy.make(FragmentType, allowed_css_classes="aaa,bbb")
        fragment1 = mommy.make(Fragment, name="azerty", type=fragment_type1)
        fragment2 = mommy.make(Fragment, name="qwerty", type=fragment_type2)

        data = {
            'form-0-id': fragment1.id,
            'form-0-type': fragment1.type.id,
            'form-0-name': fragment1.name+"!",
            'form-0-css_class': ["oups"],
            'form-0-position': 5,
            'form-0-delete_me': False,

            'form-1-id': fragment2.id,
            'form-1-type': fragment2.type.id,
            'form-1-name': fragment2.name+"+",
            'form-1-css_class': ["aaa", "bbb"],
            'form-1-position': 2,
            'form-1-delete_me': False,

            'form-TOTAL_FORMS': 2,
            'form-INITIAL_FORMS': 2,
            'form-MAX_NUM_FORMS': 2
        }

        self._log_as_editor()

        url = reverse("coop_cms_edit_fragments")
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(200, response.status_code)

        soup = BeautifulSoup(response.content)
        errs = soup.select("ul.errorlist li")
        self.assertEqual([], errs)

        expected = u'<script>$.colorbox.close(); window.location=window.location;</script>'.format()
        self.assertEqual(response.content, expected)

        self.assertEqual(2, Fragment.objects.count())
        fragment1 = Fragment.objects.get(id=fragment1.id)
        fragment2 = Fragment.objects.get(id=fragment2.id)

        self.assertEqual(fragment1.type, fragment_type1)
        self.assertEqual(fragment1.name, "azerty!")
        self.assertEqual(fragment1.css_class, "oups")
        self.assertEqual(fragment1.position, 5)

        self.assertEqual(fragment2.type, fragment_type2)
        self.assertEqual(fragment2.name, "qwerty+")
        self.assertEqual(fragment2.css_class, "aaa bbb")
        self.assertEqual(fragment2.position, 2)

    def test_edit_fragment_css_not_allowed(self):
        """edit fragment: invalid css"""

        template = settings.COOP_CMS_ARTICLE_TEMPLATES[0][0]
        get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)
        
        fragment_type1 = mommy.make(FragmentType, allowed_css_classes="")
        fragment_type2 = mommy.make(FragmentType)
        fragment1 = mommy.make(Fragment, name="azerty", type=fragment_type1)
        fragment2 = mommy.make(Fragment, name="qwerty", type=fragment_type2)
        
        data = {
            'form-0-id': fragment1.id,
            'form-0-type': fragment1.type.id,
            'form-0-name': fragment1.name+"!",
            'form-0-css_class': "oups",
            'form-0-position': 5,
            'form-0-delete_me': False,
            
            'form-1-id': fragment2.id,
            'form-1-type': fragment2.type.id,
            'form-1-name': fragment2.name+"+",
            'form-1-css_class': "aaa",
            'form-1-position': 2,
            'form-1-delete_me': False,
            
            'form-TOTAL_FORMS': 2,
            'form-INITIAL_FORMS': 2,
            'form-MAX_NUM_FORMS': 2
        }
        
        self._log_as_editor()
        
        url = reverse("coop_cms_edit_fragments")
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(200, response.status_code)
        
        soup = BeautifulSoup(response.content)
        errs = soup.select("ul.errorlist li")
        self.assertEqual(2, len(errs))
        
        self.assertEqual(2, Fragment.objects.count())
        fragment1 = Fragment.objects.get(id=fragment1.id)
        fragment2 = Fragment.objects.get(id=fragment2.id)

        self.assertEqual(fragment1.type, fragment_type1)
        self.assertEqual(fragment1.name, "azerty")
        self.assertEqual(fragment1.css_class, "")

        self.assertEqual(fragment2.type, fragment_type2)
        self.assertEqual(fragment2.name, "qwerty")
        self.assertEqual(fragment2.css_class, "")

    def test_edit_fragment_css_not_allowed2(self):
        """edit fragment: invalid css for only 1"""

        template = settings.COOP_CMS_ARTICLE_TEMPLATES[0][0]
        get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)

        fragment_type1 = mommy.make(FragmentType, allowed_css_classes="aaa")
        fragment_type2 = mommy.make(FragmentType)
        fragment1 = mommy.make(Fragment, name="azerty", type=fragment_type1)
        fragment2 = mommy.make(Fragment, name="qwerty", type=fragment_type2)

        data = {
            'form-0-id': fragment1.id,
            'form-0-type': fragment1.type.id,
            'form-0-name': fragment1.name+"!",
            'form-0-css_class': "oups",
            'form-0-position': 5,
            'form-0-delete_me': False,

            'form-1-id': fragment2.id,
            'form-1-type': fragment2.type.id,
            'form-1-name': fragment2.name+"+",
            'form-1-css_class': "aaa",
            'form-1-position': 2,
            'form-1-delete_me': False,

            'form-TOTAL_FORMS': 2,
            'form-INITIAL_FORMS': 2,
            'form-MAX_NUM_FORMS': 2
        }

        self._log_as_editor()

        url = reverse("coop_cms_edit_fragments")
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(200, response.status_code)

        soup = BeautifulSoup(response.content)
        errs = soup.select("ul.errorlist li")
        self.assertEqual(2, len(errs))

        self.assertEqual(2, Fragment.objects.count())
        fragment1 = Fragment.objects.get(id=fragment1.id)
        fragment2 = Fragment.objects.get(id=fragment2.id)

        self.assertEqual(fragment1.type, fragment_type1)
        self.assertEqual(fragment1.name, "azerty")
        self.assertEqual(fragment1.css_class, "")

        self.assertEqual(fragment2.type, fragment_type2)
        self.assertEqual(fragment2.name, "qwerty")
        self.assertEqual(fragment2.css_class, "")

    def test_edit_fragment_css_not_allowed3(self):
        """edit fragment: one invalid css """

        template = settings.COOP_CMS_ARTICLE_TEMPLATES[0][0]
        get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)

        fragment_type1 = mommy.make(FragmentType, allowed_css_classes="aaa,bbb")
        fragment_type2 = mommy.make(FragmentType)
        fragment1 = mommy.make(Fragment, name="azerty", type=fragment_type1)
        fragment2 = mommy.make(Fragment, name="qwerty", type=fragment_type2)

        data = {
            'form-0-id': fragment1.id,
            'form-0-type': fragment1.type.id,
            'form-0-name': fragment1.name+"!",
            'form-0-css_class': ["bbb", "oups"],
            'form-0-position': 5,
            'form-0-delete_me': False,

            'form-1-id': fragment2.id,
            'form-1-type': fragment2.type.id,
            'form-1-name': fragment2.name+"+",
            'form-1-css_class': "aaa",
            'form-1-position': 2,
            'form-1-delete_me': False,

            'form-TOTAL_FORMS': 2,
            'form-INITIAL_FORMS': 2,
            'form-MAX_NUM_FORMS': 2
        }

        self._log_as_editor()

        url = reverse("coop_cms_edit_fragments")
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(200, response.status_code)

        soup = BeautifulSoup(response.content)
        errs = soup.select("ul.errorlist li")
        self.assertEqual(2, len(errs))

        self.assertEqual(2, Fragment.objects.count())
        fragment1 = Fragment.objects.get(id=fragment1.id)
        fragment2 = Fragment.objects.get(id=fragment2.id)

        self.assertEqual(fragment1.type, fragment_type1)
        self.assertEqual(fragment1.name, "azerty")
        self.assertEqual(fragment1.css_class, "")

        self.assertEqual(fragment2.type, fragment_type2)
        self.assertEqual(fragment2.name, "qwerty")
        self.assertEqual(fragment2.css_class, "")

    def test_edit_fragment_delete(self):
        """delete fragment"""
        template = settings.COOP_CMS_ARTICLE_TEMPLATES[0][0]
        get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)
        
        fragment_type1 = mommy.make(FragmentType)
        fragment_type2 = mommy.make(FragmentType)
        fragment1 = mommy.make(Fragment, name="azerty", type=fragment_type1)
        fragment2 = mommy.make(Fragment, name="qwerty", type=fragment_type2)
        
        data = {
            'form-0-id': fragment1.id,
            'form-0-type': fragment1.type.id,
            'form-0-name': fragment1.name+"!",
            'form-0-css_class': "",
            'form-0-position': 5,
            'form-0-delete_me': False,
            
            'form-1-id': fragment2.id,
            'form-1-type': fragment2.type.id,
            'form-1-name': fragment2.name+"+",
            'form-1-css_class': "",
            'form-1-position': 2,
            'form-1-delete_me': True,
            
            'form-TOTAL_FORMS': 2,
            'form-INITIAL_FORMS': 2,
            'form-MAX_NUM_FORMS': 2
        }
        
        self._log_as_editor()
        
        url = reverse("coop_cms_edit_fragments")
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(200, response.status_code)
        
        soup = BeautifulSoup(response.content)
        errs = soup.select("ul.errorlist li")
        self.assertEqual([], errs)
        
        expected = u'<script>$.colorbox.close(); window.location=window.location;</script>'.format()
        self.assertEqual(response.content, expected)
        
        self.assertEqual(1, Fragment.objects.count())
        fragment1 = Fragment.objects.get(id=fragment1.id)
        self.assertEqual(Fragment.objects.filter(id=fragment2.id).count(), 0)

        self.assertEqual(fragment1.type, fragment_type1)
        self.assertEqual(fragment1.name, "azerty!")
        self.assertEqual(fragment1.css_class, "")
        self.assertEqual(fragment1.position, 5)
        
    def test_edit_fragment_invalid_position(self):
        """edit fragment: invalid pos"""

        template = settings.COOP_CMS_ARTICLE_TEMPLATES[0][0]
        get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)
        
        fragment_type1 = mommy.make(FragmentType)
        fragment_type2 = mommy.make(FragmentType)
        fragment1 = mommy.make(Fragment, name="azerty", type=fragment_type1)
        fragment2 = mommy.make(Fragment, name="qwerty", type=fragment_type2)
        
        data = {
            'form-0-id': fragment1.id,
            'form-0-type': fragment1.type.id,
            'form-0-name': fragment1.name+"!",
            'form-0-css_class': "",
            'form-0-position': "AAA",
            'form-0-delete_me': False,
            
            'form-1-id': fragment2.id,
            'form-1-type': fragment2.type.id,
            'form-1-name': fragment2.name+"+",
            'form-1-css_class': "",
            'form-1-position': 2,
            'form-1-delete_me': False,
            
            'form-TOTAL_FORMS': 2,
            'form-INITIAL_FORMS': 2,
            'form-MAX_NUM_FORMS': 2
        }
        
        self._log_as_editor()
        
        url = reverse("coop_cms_edit_fragments")
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(200, response.status_code)
        
        soup = BeautifulSoup(response.content)
        errs = soup.select("ul.errorlist li")
        self.assertEqual(1, len(errs))
    
    def test_edit_fragment_empty_name(self):
        """edit fragment empty name"""

        template = settings.COOP_CMS_ARTICLE_TEMPLATES[0][0]
        get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)
        
        fragment_type1 = mommy.make(FragmentType)
        fragment_type2 = mommy.make(FragmentType)
        fragment1 = mommy.make(Fragment, name="azerty", type=fragment_type1)
        fragment2 = mommy.make(Fragment, name="qwerty", type=fragment_type2)
        
        data = {
            'form-0-id': fragment1.id,
            'form-0-type': fragment1.type.id,
            'form-0-name': "",
            'form-0-css_class': "",
            'form-0-position': 1,
            'form-0-delete_me': False,
            
            'form-1-id': fragment2.id,
            'form-1-type': fragment2.type.id,
            'form-1-name': fragment2.name+"+",
            'form-1-css_class': "",
            'form-1-position': 2,
            'form-1-delete_me': False,
            
            'form-TOTAL_FORMS': 2,
            'form-INITIAL_FORMS': 2,
            'form-MAX_NUM_FORMS': 2
        }
        
        self._log_as_editor()
        
        url = reverse("coop_cms_edit_fragments")
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(200, response.status_code)
        
        soup = BeautifulSoup(response.content)
        errs = soup.select("ul.errorlist li")
        self.assertEqual(1, len(errs))

    def test_edit_fragment_permission_denied(self):
        """edit fragment: not allowed"""

        template = settings.COOP_CMS_ARTICLE_TEMPLATES[0][0]
        get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)
        
        fragment_type1 = mommy.make(FragmentType)
        fragment_type2 = mommy.make(FragmentType)
        fragment1 = mommy.make(Fragment, name="azerty", type=fragment_type1)
        fragment2 = mommy.make(Fragment, name="qwerty", type=fragment_type2)
        
        data = {
            'form-0-id': fragment1.id,
            'form-0-type': fragment1.type.id,
            'form-0-name': fragment1.name+"!",
            'form-0-css_class': "",
            'form-0-position': 5,
            'form-0-delete_me': False,
            
            'form-1-id': fragment2.id,
            'form-1-type': fragment2.type.id,
            'form-1-name': fragment2.name+"+",
            'form-1-css_class': "",
            'form-1-position': 2,
            'form-1-delete_me': False,
            
            'form-TOTAL_FORMS': 2,
            'form-INITIAL_FORMS': 2,
            'form-MAX_NUM_FORMS': 2
        }
        
        url = reverse("coop_cms_edit_fragments")
        response = self.client.post(url, data=data, follow=False)
        
        self.assertEqual(302, response.status_code)
        
        self.assertEqual(2, Fragment.objects.count())
        fragment1 = Fragment.objects.get(id=fragment1.id)
        fragment2 = Fragment.objects.get(id=fragment2.id)

        self.assertEqual(fragment1.type, fragment_type1)
        self.assertEqual(fragment1.name, "azerty")
        self.assertEqual(fragment1.css_class, "")
        self.assertEqual(fragment1.position, 1)
    
        self.assertEqual(fragment2.type, fragment_type2)
        self.assertEqual(fragment2.name, "qwerty")
        self.assertEqual(fragment2.css_class, "")
        self.assertEqual(fragment2.position, 1)

        self._log_as_regular_user()
        response = self.client.post(url, data=data)
        self.assertEqual(403, response.status_code)  
        
        self.assertEqual(2, Fragment.objects.count())
        fragment1 = Fragment.objects.get(id=fragment1.id)
        fragment2 = Fragment.objects.get(id=fragment2.id)

        self.assertEqual(fragment1.type, fragment_type1)
        self.assertEqual(fragment1.name, "azerty")
        self.assertEqual(fragment1.css_class, "")
        self.assertEqual(fragment1.position, 1)
    
        self.assertEqual(fragment2.type, fragment_type2)
        self.assertEqual(fragment2.name, "qwerty")
        self.assertEqual(fragment2.css_class, "")
        self.assertEqual(fragment2.position, 1)



# -*- coding: utf-8 -*-
""""""

from django.conf import settings
from django.core.management.base import BaseCommand
from django.db import connection

from coop_cms.models import Alias
from coop_cms.settings import is_localized


class Command(BaseCommand):
    """patch alias redirect"""
    help = u"patch alias redirect"
    use_argparse = False

    def handle(self, *args, **options):
        """command"""
        #look for emailing to be sent
        verbose = options.get('verbosity', 1)
        
        if not is_localized():
            print "the site is not localized this is not required"
        
        from modeltranslation.utils import build_localized_fieldname
        
        for alias in Alias.objects.all():
            cursor = connection.cursor()
            
            cursor.execute(
                '''SELECT path, redirect_url FROM coop_cms_alias where id={0}'''.format(alias.id))
            row = cursor.fetchone()
            print row
            (path, redirect_url) = row
            
            languages = [x for (x, y) in settings.LANGUAGES]
            
            lang_code = languages[0]
            path_field_name = build_localized_fieldname('path', lang_code)
            redirect_url_field_name = build_localized_fieldname('redirect_url', lang_code)
            if (not getattr(alias, path_field_name)) and (not getattr(alias, redirect_url_field_name)):
                print "update", alias.id, path, redirect_url
                setattr(alias, path_field_name, path)
                setattr(alias, redirect_url_field_name, redirect_url)
                alias.save()



# -*- coding:utf-8 -*-
"""
urls
"""

from django.conf.urls import patterns, url

from coop_cms.apps.rss_sync import views

urlpatterns = patterns('',
    url(r'^collect-rss-items/(?P<source_id>\d+)$', views.collect_rss_items_view, name='rss_sync_collect_rss_items'),
    url(r'^create-cms-article/(?P<item_id>\d+)$', views.create_cms_article_view, name='rss_sync_create_cms_article'),
)



# -*- coding: utf-8 -*-
"""
Utilities for Bootstrap CSS framework
"""

from django import VERSION

if VERSION > (1, 7, 0):
    from django.apps import AppConfig

    class CoopBootstrapAppConfig(AppConfig):
        name = 'coop_cms.apps.coop_bootstrap'
        verbose_name = "coop CMS > Support of Bootstrap CSS framework"



# -*- coding: utf-8 -*-
""""""

from django.core.exceptions import PermissionDenied
from django.core.urlresolvers import reverse, resolve, NoReverseMatch
from django.contrib.auth.views import redirect_to_login

from coop_cms.utils import get_login_url


class PermissionsMiddleware(object):
    """Handle permission"""

    def process_exception(self, request, exception):
        """manage exception"""

        if isinstance(exception, PermissionDenied) and (not request.user.is_authenticated()):
            try:
                login_url = get_login_url()
            except NoReverseMatch:
                login_url = None
            return redirect_to_login(request.path, login_url)



# -*- coding: utf-8 -*-

from django.conf import settings

from datetime import datetime

from django.contrib.contenttypes.models import ContentType
from django.contrib.sites.models import Site
from django.core.urlresolvers import reverse

from model_mommy import mommy

from coop_cms.models import NavNode, BaseArticle, ArticleCategory
from coop_cms.settings import (
    is_localized, get_article_class, get_article_templates, get_navtree_class, is_perm_middleware_installed,
)
from coop_cms.tests import BaseArticleTest, BeautifulSoup, make_dt
from coop_cms.utils import get_login_url


class ArticleTest(BaseArticleTest):
    
    def setUp(self):
        super(ArticleTest, self).setUp()
        self._default_article_templates = settings.COOP_CMS_ARTICLE_TEMPLATES
        settings.COOP_CMS_ARTICLE_TEMPLATES = (
            ('test/newsletter_red.html', 'Red'),
            ('test/newsletter_blue.html', 'Blue'),
        )
        self._HTML_EDITOR_LINK_MODELS = getattr(settings, 'HTML_EDITOR_LINK_MODELS', [])
        article_class = get_article_class()
        content_type = ContentType.objects.get_for_model(article_class)
        settings.HTML_EDITOR_LINK_MODELS = ['{0}.{1}'.format(content_type.app_label, content_type.model)]
        
    def tearDown(self):
        super(ArticleTest, self).tearDown()
        # restore
        settings.COOP_CMS_ARTICLE_TEMPLATES = self._default_article_templates
        settings.HTML_EDITOR_LINK_MODELS = self._HTML_EDITOR_LINK_MODELS

    def _check_article(self, response, data):
        for (key, value) in data.items():
            self.assertContains(response, value)
            
    def _check_article_not_changed(self, article, data, initial_data):
        article = get_article_class().objects.get(id=article.id)

        for (key, value) in data.items():
            self.assertNotEquals(getattr(article, key), value)
            
        for (key, value) in initial_data.items():
            self.assertEquals(getattr(article, key), value)

    def test_view_article(self):
        article = get_article_class().objects.create(title="test", publication=BaseArticle.PUBLISHED)
        self.assertEqual(article.slug, 'test')
        response = self.client.get(article.get_absolute_url())
        self.assertEqual(200, response.status_code)

    def test_view_article_full_slug(self):
        article = get_article_class().objects.create(title="When I am 64", publication=BaseArticle.PUBLISHED)
        self.assertEqual(article.slug, 'when-i-am-64')
        response = self.client.get(article.get_absolute_url())
        self.assertEqual(200, response.status_code)
        
    def test_publication_flag_published(self):
        article = get_article_class().objects.create(title="test", publication=BaseArticle.PUBLISHED)
        self.assertEqual(article.is_draft(), False)
        response = self.client.get(article.get_absolute_url())
        self.assertEqual(200, response.status_code)
        
    def test_publication_flag_archived(self):
        article = get_article_class().objects.create(title="test", publication=BaseArticle.ARCHIVED)
        self.assertEqual(article.is_draft(), False)
        response = self.client.get(article.get_absolute_url())
        self.assertEqual(404, response.status_code)
        
    def test_publication_flag_draft(self):
        article = get_article_class().objects.create(title="test", publication=BaseArticle.DRAFT)
        self.assertEqual(article.is_draft(), True)
        url = article.get_absolute_url()
        response = self.client.get(url)
        if is_perm_middleware_installed():
            self.assertEqual(302, response.status_code)
            auth_url = get_login_url()
            self.assertRedirects(response, auth_url+'?next='+url)
        else:
            self.assertEqual(403, response.status_code)

    def test_login_required_authenticated(self):
        """show page if permission required and authenticated"""
        self._log_as_non_editor()
        article = get_article_class().objects.create(
            title="test", publication=BaseArticle.PUBLISHED, login_required=True
        )
        url = article.get_absolute_url()
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)

    def test_login_required_not_authenticated(self):
        """raise permission denied if permission required not authenticated"""
        self.client.logout()
        article = get_article_class().objects.create(
            title="test", publication=BaseArticle.PUBLISHED, login_required=True
        )
        url = article.get_absolute_url()
        response = self.client.get(url)
        if is_perm_middleware_installed():
            self.assertEqual(302, response.status_code)
            auth_url = get_login_url()
            self.assertRedirects(response, auth_url+'?next='+url)
        else:
            self.assertEqual(403, response.status_code)
        
    def test_404_ok(self):
        response = self.client.get("/jhjhjkahekhj", follow=True)
        self.assertEqual(404, response.status_code)
        
    def test_is_navigable(self):
        article = get_article_class().objects.create(title="test", publication=BaseArticle.PUBLISHED)
        if is_localized():
            lang = settings.LANGUAGES[0][0]
            self.assertEqual('/{0}/test/'.format(lang), article.get_absolute_url())
        else:
            self.assertEqual('/test/', article.get_absolute_url())

    def test_create_slug(self):
        article = get_article_class().objects.create(title=u"voici l'été", publication=BaseArticle.PUBLISHED)
        self.assertEqual(article.slug, 'voici-lete')
        response = self.client.get(article.get_absolute_url())
        self.assertEqual(200, response.status_code)
        
    def test_edit_article(self):
        article = get_article_class().objects.create(title="test", publication=BaseArticle.PUBLISHED)
        
        data = {"title": 'salut', 'content': 'bonjour!'}
        
        self._log_as_editor()
        response = self.client.post(article.get_edit_url(), data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        self._check_article(response, data)
        
        data = {"title": 'bye', 'content': 'au revoir'}
        response = self.client.post(article.get_edit_url(), data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        self._check_article(response, data)

    def test_edit_article_html(self):
        """It should save article correctly"""
        article = get_article_class().objects.create(title="test", publication=BaseArticle.PUBLISHED)

        data = {"title": 'salut', 'content': '<h2>Hello</h2><p>Hello</p>'}

        self._log_as_editor()
        response = self.client.post(article.get_edit_url(), data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        self._check_article(response, data)

    def test_edit_article_html_with_br(self):
        """It should save article correctly"""
        article = get_article_class().objects.create(title="test", publication=BaseArticle.PUBLISHED)

        data = {"title": 'salut', 'content': '<h2>Hello</h2><p>Hello<br>Cool<br /></p>'}

        self._log_as_editor()
        response = self.client.post(article.get_edit_url(), data=data, follow=True)

        self.assertEqual(response.status_code, 200)
        expected_content = '<h2>Hello</h2><p>Hello<br />Cool<br /></p>'
        data['content'] = expected_content
        self._check_article(response, data)

    def test_edit_article_html_with_img_wrapper(self):
        """It should save article correctly"""
        article = get_article_class().objects.create(title="test", publication=BaseArticle.PUBLISHED)

        content = """
            <p><div
            contenteditable="false"
            style="overflow: hidden; position: relative; display: inline-block; float: none;"
            class="ui-wrapper aloha-image-box-active Aloha_Image_Resize aloha"
            >
                <img src="moby-dick.jpg" style="height: 207px; width: 269px;" class="ui-resizable" />
                <div class="ui-resizable-handle ui-resizable-ne" style="z-index: 1000;"></div>
                <div class="ui-resizable-handle ui-resizable-se ui-icon" style="z-index: 1000;"></div>
                <div class="ui-resizable-handle ui-resizable-sw" style="z-index: 1000;"></div>
                <div class="ui-resizable-handle ui-resizable-nw" style="z-index: 1000;"></div>
            </div></p>
        """

        data = {"title": 'salut', 'content': content}

        self._log_as_editor()
        response = self.client.post(article.get_edit_url(), data=data, follow=True)

        self.assertEqual(response.status_code, 200)

        expected_content = '<p><img class="" src="moby-dick.jpg" style="height: 207px; width: 269px;"/></p>'
        data['content'] = expected_content
        self._check_article(response, data)

    def test_edit_article_draft(self):
        article = get_article_class().objects.create(title="test", publication=BaseArticle.DRAFT)
        
        data = {"title": 'salut', 'content': 'bonjour!'}
        
        self._log_as_editor()
        response = self.client.post(article.get_edit_url(), data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        self._check_article(response, data)
        
        data = {"title": 'bye', 'content': 'au revoir'}
        response = self.client.post(article.get_edit_url(), data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        self._check_article(response, data)
        
    def test_edit_article_archived(self):
        article = get_article_class().objects.create(title="test", publication=BaseArticle.ARCHIVED)
        
        data = {"title": 'salut', 'content': 'bonjour!'}
        
        self._log_as_editor()
        response = self.client.post(article.get_edit_url(), data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        self._check_article(response, data)
        
        data = {"title": 'bye', 'content': 'au revoir'}
        response = self.client.post(article.get_edit_url(), data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        self._check_article(response, data)
        
    def test_post_on_view_article(self):
        initial_data = {'title': "test", 'content': "this is my article content"}
        article = get_article_class().objects.create(publication=BaseArticle.PUBLISHED, **initial_data)
        
        data = {"title": 'salut', 'content': 'bonjour!'}
        
        self._log_as_editor()
        response = self.client.post(article.get_absolute_url(), data=data, follow=True)
        self.assertEqual(response.status_code, 404)
        
        article = get_article_class().objects.get(id=article.id)
        self.assertEquals(article.title, initial_data['title'])
        self.assertEquals(article.content, initial_data['content'])
        
    def test_article_edition_permission(self):
        initial_data = {'title': "test", 'content': "this is my article content"}
        article = get_article_class().objects.create(publication=BaseArticle.PUBLISHED, **initial_data)
        url = article.get_edit_url()
        data = {"title": 'salut', "content": 'oups'}
        response = self.client.post(url, data=data)
        if is_perm_middleware_installed():
            self.assertEqual(302, response.status_code)
            auth_url = get_login_url()
            self.assertRedirects(response, auth_url+'?next='+url)
        else:
            self.assertEqual(403, response.status_code)
        
        article = get_article_class().objects.get(id=article.id)
        self.assertEquals(article.title, initial_data['title'])
        self.assertEquals(article.content, initial_data['content'])
        
    def _is_inline_html_editor_found(self, response):
        self.assertEqual(200, response.status_code)
        inline_editor_init_url = reverse('html_editor_init')
        content = unicode(response.content, 'utf-8')
        return content.find(inline_editor_init_url) > 0
        
    def test_edit_permission(self):
        initial_data = {'title': "ceci est un test", 'content': "this is my article content"}
        article = get_article_class().objects.create(publication=BaseArticle.PUBLISHED, **initial_data)
        response = self.client.get(article.get_absolute_url(), follow=True)
        self.assertEqual(200, response.status_code)
        
        url = article.get_edit_url()
        response = self.client.get(url, follow=False)
        if is_perm_middleware_installed():
            self.assertEqual(302, response.status_code)
            auth_url = get_login_url()
            self.assertRedirects(response, auth_url+'?next='+url)
        else:
            self.assertEqual(403, response.status_code)
    
        self._log_as_editor()
        response = self.client.get(article.get_edit_url(), follow=False)
        self.assertEqual(200, response.status_code)
        
    def test_inline_html_editor_loaded(self):
        initial_data = {'title': u"ceci est un test", 'content': u"this is my article content"}
        article = get_article_class().objects.create(publication=BaseArticle.PUBLISHED, **initial_data)
        response = self.client.get(article.get_absolute_url())
        self.assertFalse(self._is_inline_html_editor_found(response))
        
        self._log_as_editor()
        response = self.client.get(article.get_edit_url())
        self.assertTrue(self._is_inline_html_editor_found(response))
        
    def test_inline_html_editor_links(self):
        slugs = ("un", "deux", "trois", "quatre")
        for slug in slugs:
            get_article_class().objects.create(publication=BaseArticle.PUBLISHED, title=slug)
        initial_data = {'title': "test", 'content': "this is my article content"}
        get_article_class().objects.create(**initial_data)
        
        self._log_as_editor()
        response = self.client.get(reverse('html_editor_init'))
        
        context_slugs = [article.slug for article in response.context['links']]
        for slug in slugs:
            self.assertTrue(slug in context_slugs)
        
    def test_view_draft_article(self):
        self.client.logout()
        article = get_article_class().objects.create(title="test", publication=BaseArticle.DRAFT)
        url = article.get_absolute_url()
        response = self.client.get(url)
        if is_perm_middleware_installed():
            self.assertEqual(302, response.status_code)
            auth_url = get_login_url()
            self.assertRedirects(response, auth_url+'?next='+url)
        else:
            self.assertEqual(403, response.status_code)
        
        self._log_as_editor()
        response = self.client.get(article.get_absolute_url())
        self.assertEqual(200, response.status_code)
        
    def test_accept_regular_html(self):
        article = get_article_class().objects.create(title="test", publication=BaseArticle.PUBLISHED)
        html = '<h1>paul</h1><a href="/" target="_blank">georges</a><p><b>ringo</b></p>'
        html += '<h6>john</h6><img src="/img.jpg"><br><table><tr><th>A</th><td>B</td></tr>'
        data = {'content': html, 'title': 'ok'}
        
        self._log_as_editor()
        response = self.client.post(article.get_edit_url(), data=data, follow=False)
        self.assertEqual(response.status_code, 302)
        
        response = self.client.get(article.get_absolute_url())
        self.assertEqual(response.status_code, 200)
        
        # checking html content would not work. Check that the article is updated
        for b in ['paul', 'georges', 'ringo', 'john']:
            self.assertContains(response, b)

    def test_publish_article(self):
        initial_data = {'title': "test", 'content': "this is my article content"}
        article = get_article_class().objects.create(publication=BaseArticle.DRAFT, **initial_data)
        
        self._log_as_editor()
        
        data = {
            'publication': BaseArticle.PUBLISHED,
        }
        
        response = self.client.post(article.get_publish_url(), data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        
        article = get_article_class().objects.get(id=article.id)
        self.assertEqual(article.title, initial_data['title'])
        self.assertEqual(article.content, initial_data['content'])
        self.assertEqual(article.publication, BaseArticle.PUBLISHED)

    def test_draft_article(self):
        initial_data = {'title': "test", 'content': "this is my article content"}
        article = get_article_class().objects.create(publication=BaseArticle.PUBLISHED, **initial_data)
        
        self._log_as_editor()
        
        data = {
            'publication': BaseArticle.DRAFT,
        }
        
        response = self.client.post(article.get_publish_url(), data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        
        article = get_article_class().objects.get(id=article.id)
        self.assertEqual(article.title, initial_data['title'])
        self.assertEqual(article.content, initial_data['content'])
        self.assertEqual(article.publication, BaseArticle.DRAFT)
        
    def test_new_article(self):
        article_class = get_article_class()
        
        self._log_as_editor()
        data = {
            'title': "Un titre",
            'publication': BaseArticle.DRAFT,
            'template': get_article_templates(None, self.user)[0][0],
            'navigation_parent': None,
            'sites': [settings.SITE_ID]
        }
        
        response = self.client.post(reverse('coop_cms_new_article'), data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        
        self.assertEqual(article_class.objects.count(), 1)
        article = article_class.objects.all()[0]
        
        self.assertEqual(article.title, data['title'])
        self.assertEqual(article.publication, data['publication'])
        self.assertEqual(article.template, data['template'])
        self.assertEqual(article.navigation_parent, None)
        self.assertEqual(NavNode.objects.count(), 0)
        self.assertEqual([a.id for a in article.sites.order_by("id")], data['sites'])

    def test_view_new_article(self):
        other_site = mommy.make(Site)

        category1 = mommy.make(ArticleCategory)
        category1.sites.clear()
        category1.sites.add(Site.objects.get_current())
        category1.save()

        category2 = mommy.make(ArticleCategory)
        category2.sites.clear()
        category2.sites.add(Site.objects.get_current())
        category2.sites.add(other_site)
        category2.save()

        category3 = mommy.make(ArticleCategory)
        category3.sites.clear()
        category3.sites.add(other_site)
        category3.save()

        category4 = mommy.make(ArticleCategory)
        category4.sites.clear()
        category4.save()

        self._log_as_editor()

        response = self.client.get(reverse('coop_cms_new_article'))
        self.assertEqual(response.status_code, 200)
        soup = BeautifulSoup(response.content)

        categories_select_node = soup.select("select#id_category")[0]

        categories = [node.text for node in categories_select_node.select('option')]

        self.assertEqual(3, len(categories))
        self.assertTrue('---------' in categories)
        self.assertTrue(category1.name in categories)
        self.assertTrue(category2.name in categories)
        self.assertTrue(category3.name not in categories)
        self.assertTrue(category4.name not in categories)

    def test_new_article_two_sites(self):
        other_site = mommy.make(Site)
        article_class = get_article_class()

        self._log_as_editor()
        data = {
            'title': "Un titre",
            'publication': BaseArticle.DRAFT,
            'template': get_article_templates(None, self.user)[0][0],
            'navigation_parent': None,
            'sites': sorted([settings.SITE_ID, other_site.id])
        }

        response = self.client.post(reverse('coop_cms_new_article'), data=data, follow=True)
        self.assertEqual(response.status_code, 200)

        self.assertEqual(article_class.objects.count(), 1)
        article = article_class.objects.all()[0]

        self.assertEqual(article.title, data['title'])
        self.assertEqual(article.publication, data['publication'])
        self.assertEqual(article.template, data['template'])
        self.assertEqual(article.navigation_parent, None)
        self.assertEqual(NavNode.objects.count(), 0)
        self.assertEqual([_article.id for _article in article.sites.order_by("id")], data['sites'])

    def test_new_article_without_site(self):
        article_class = get_article_class()

        self._log_as_editor()
        data = {
            'title': "Un titre",
            'publication': BaseArticle.DRAFT,
            'template': get_article_templates(None, self.user)[0][0],
            'navigation_parent': None,
            'sites': []
        }

        response = self.client.post(reverse('coop_cms_new_article'), data=data, follow=True)
        self.assertEqual(response.status_code, 200)

        self.assertEqual(article_class.objects.count(), 0)

        soup = BeautifulSoup(response.content)
        self.assertEqual(len(soup.select("ul.errorlist")), 1)

    def test_new_article_invalid_site(self):
        article_class = get_article_class()

        self._log_as_editor()
        data = {
            'title': "Un titre",
            'publication': BaseArticle.DRAFT,
            'template': get_article_templates(None, self.user)[0][0],
            'navigation_parent': None,
            'sites': [settings.SITE_ID, 999]
        }

        response = self.client.post(reverse('coop_cms_new_article'), data=data, follow=True)
        self.assertEqual(response.status_code, 200)

        self.assertEqual(article_class.objects.count(), 0)

        soup = BeautifulSoup(response.content)
        self.assertEqual(len(soup.select("ul.errorlist")), 1)

    def test_new_article_invalid_category_site(self):
        article_class = get_article_class()

        category1 = mommy.make(ArticleCategory)
        category1.sites.clear()
        category1.sites.add(Site.objects.get_current())
        category1.save()

        category2 = mommy.make(ArticleCategory)
        category2.sites.clear()
        category2.save()

        self._log_as_editor()
        data = {
            'title': "Un titre",
            'category': category2.id,
            'publication': BaseArticle.DRAFT,
            'template': get_article_templates(None, self.user)[0][0],
            'navigation_parent': None,
            'sites': [settings.SITE_ID]
        }

        response = self.client.post(reverse('coop_cms_new_article'), data=data, follow=True)
        self.assertEqual(response.status_code, 200)

        self.assertEqual(article_class.objects.count(), 0)

        soup = BeautifulSoup(response.content)
        self.assertEqual(len(soup.select("ul.errorlist")), 1)

    def test_new_article_title_required(self):
        article_class = get_article_class()

        self._log_as_editor()
        data = {
            'title': "",
            'publication': BaseArticle.DRAFT,
            'template': get_article_templates(None, self.user)[0][0],
            'navigation_parent': None,
            'sites': [settings.SITE_ID]
        }
        
        response = self.client.post(reverse('coop_cms_new_article'), data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        soup = BeautifulSoup(response.content)
        
        self.assertEqual(article_class.objects.count(), 0)
        self.assertEqual(len(soup.select("ul.errorlist")), 1)
        
    def test_new_article_published(self):
        article_class = get_article_class()
        
        self._log_as_editor()
        data = {
            'title': "Un titre",
            'publication': BaseArticle.PUBLISHED,
            'template': get_article_templates(None, self.user)[0][0],
            'navigation_parent': None,
            'sites': [settings.SITE_ID]
        }
        
        response = self.client.post(reverse('coop_cms_new_article'), data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        
        self.assertEqual(article_class.objects.count(), 1)
        article = article_class.objects.all()[0]
        
        self.assertEqual(article.title, data['title'])
        self.assertEqual(article.publication, data['publication'])
        self.assertEqual(article.template, data['template'])
        self.assertEqual(article.navigation_parent, None)
        self.assertEqual(NavNode.objects.count(), 0)
        
    def test_new_article_anonymous(self):
        article_class = get_article_class()
        
        self._log_as_editor() #create self.user
        self.client.logout()
        data = {
            'title': "Ceci est un titre",
            'publication': BaseArticle.DRAFT,
            'template': get_article_templates(None, self.user)[0][0],
            'navigation_parent': None,
            'sites': [settings.SITE_ID]
        }
        
        response = self.client.post(reverse('coop_cms_new_article'), data=data, follow=True)
        self.assertEqual(200, response.status_code) #if can_edit returns 404 error
        next_url = response.redirect_chain[-1][0]
        login_url = reverse('django.contrib.auth.views.login')
        self.assertTrue(login_url in next_url)
        
        self.assertEqual(article_class.objects.filter(title=data['title']).count(), 0)
        
    def test_new_article_no_perm(self):
        article_class = get_article_class()
        
        self._log_as_editor_no_add()
        data = {
            'title': "Ceci est un titre",
            'publication': BaseArticle.DRAFT,
            'template': get_article_templates(None, self.user)[0][0],
            'navigation_parent': None,
            'sites': [settings.SITE_ID]
        }
        
        response = self.client.post(reverse('coop_cms_new_article'), data=data, follow=True)
        self.assertEqual(403, response.status_code)
        self.assertEqual(article_class.objects.filter(title=data['title']).count(), 0)
        
    def test_new_article_navigation(self):
        article_class = get_article_class()
        
        tree = get_navtree_class().objects.create()
        
        self._log_as_editor()
        data = {
            'title': "Un titre",
            'publication': BaseArticle.PUBLISHED,
            'template': get_article_templates(None, self.user)[0][0],
            'navigation_parent': -tree.id,
            'sites': [settings.SITE_ID]
        }
        
        response = self.client.post(reverse('coop_cms_new_article'), data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        
        self.assertEqual(article_class.objects.count(), 1)
        article = article_class.objects.all()[0]
        
        self.assertEqual(article.title, data['title'])
        self.assertEqual(article.publication, data['publication'])
        self.assertEqual(article.template, data['template'])
        self.assertEqual(article.navigation_parent, -tree.id)
        
        self.assertEqual(NavNode.objects.count(), 1)
        node = NavNode.objects.all()[0]
        self.assertEqual(node.content_object, article)
        self.assertEqual(node.parent, None)
        self.assertEqual(node.tree, tree)
        
    def test_new_article_navigation_leaf(self):
        initial_data = {'title': "test", 'content': "this is my article content"}
        article_class = get_article_class()
        art1 = article_class.objects.create(publication=BaseArticle.PUBLISHED, **initial_data)
        
        tree = get_navtree_class().objects.create()
        ct = ContentType.objects.get_for_model(article_class)
        node1 = NavNode.objects.create(content_type=ct, object_id=art1.id, tree=tree, parent=None)
        
        self._log_as_editor()
        data = {
            'title': "Un titre",
            'publication': BaseArticle.PUBLISHED,
            'template': get_article_templates(None, self.user)[0][0],
            'navigation_parent': node1.id,
            'sites': [settings.SITE_ID]
        }
        
        response = self.client.post(reverse('coop_cms_new_article'), data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(article_class.objects.exclude(id=art1.id).count(), 1)
        art2 = article_class.objects.exclude(id=art1.id)[0]
        
        self.assertEqual(art2.title, data['title'])
        self.assertEqual(art2.publication, data['publication'])
        self.assertEqual(art2.template, data['template'])
        self.assertEqual(art2.navigation_parent, node1.id)
        
        self.assertEqual(NavNode.objects.count(), 2)
        node2 = NavNode.objects.exclude(id=node1.id)[0]
        self.assertEqual(node2.content_object, art2)
        self.assertEqual(node2.parent, node1)
        
    def test_article_settings(self, move_nav=False):
        initial_data = {'title': "test", 'content': "this is my article content"}
        article_class = get_article_class()
        art0 = mommy.make(article_class, slug="art0" )
        
        art1 = article_class.objects.create(publication=BaseArticle.PUBLISHED, **initial_data)
        
        tree = get_navtree_class().objects.create()
        ct = ContentType.objects.get_for_model(article_class)
        node1 = NavNode.objects.create(content_type=ct, object_id=art0.id, tree=tree, parent=None)
        node2 = NavNode.objects.create(content_type=ct, object_id=art0.id, tree=tree, parent=None)
        
        category = mommy.make(ArticleCategory)
        
        self._log_as_editor()
        data = {
            'template': get_article_templates(None, self.user)[0][0],
            'category': category.id,
            'publication': BaseArticle.PUBLISHED,
            'publication_date': "2013-01-01 12:00:00",
            'headline': True,
            'in_newsletter': True,
            'summary': 'short summary',
            'navigation_parent': node1.id,
            'sites': [settings.SITE_ID]
        }
        
        response = self.client.post(reverse('coop_cms_article_settings', args=[art1.id]), data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        
        self.assertEqual(article_class.objects.exclude(id__in=(art1.id, art0.id)).count(), 0)
        art1 = article_class.objects.get(id=art1.id)
        
        self.assertEqual(art1.title, initial_data['title'])
        self.assertEqual(art1.publication, data['publication'])
        self.assertEqual(art1.navigation_parent, node1.id)
        self.assertEqual(art1.publication_date, make_dt(datetime(2013, 1, 1, 12, 0, 0)))
        self.assertEqual(art1.headline, data['headline'])
        self.assertEqual(art1.in_newsletter, data['in_newsletter'])
        self.assertEqual(art1.summary, data['summary'])
        self.assertEqual(art1.template, data['template'])
        self.assertEqual([a.id for a in art1.sites.all()], data['sites'])
        self.assertEqual(NavNode.objects.count(), 3)
        node = NavNode.objects.exclude(id__in=(node1.id, node2.id))[0]
        self.assertEqual(node.content_object, art1)
        self.assertEqual(node.parent, node1)
        
        # Update the article
        category2 = mommy.make(ArticleCategory)
        
        node_id = node2.id if move_nav else node1.id
        
        data = {
            'template': get_article_templates(None, self.user)[1][0],
            'category': category2.id,
            'publication': BaseArticle.DRAFT,
            'publication_date': "2013-01-01 18:00:00",
            'headline': False,
            'in_newsletter': False,
            'summary': 'another summary',
            'navigation_parent': node_id,
            'sites': [settings.SITE_ID]
        }
        
        response = self.client.post(reverse('coop_cms_article_settings', args=[art1.id]), data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        
        self.assertEqual(article_class.objects.exclude(id__in=(art1.id, art0.id)).count(), 0)
        art1 = article_class.objects.get(id=art1.id)
        
        self.assertEqual(art1.title, initial_data['title'])
        self.assertEqual(art1.publication, data['publication'])
        self.assertEqual(art1.template, data['template'])
        self.assertEqual(art1.publication_date, make_dt(datetime(2013, 1, 1, 18, 0, 0)))
        self.assertEqual(art1.headline, data['headline'])
        self.assertEqual(art1.in_newsletter, data['in_newsletter'])
        self.assertEqual(art1.summary, data['summary'])

        if move_nav:
            self.assertEqual(NavNode.objects.count(), 4)
            node = NavNode.objects.exclude(id__in=(node1.id, node2.id, node.id))[0]
            self.assertEqual(node.content_object, art1)
            self.assertEqual(node.parent, node2)
        else:
            self.assertEqual(NavNode.objects.count(), 3)
            node = NavNode.objects.exclude(id__in=(node1.id, node2.id))[0]
            self.assertEqual(node.content_object, art1)
            self.assertEqual(node.parent, node1)
            
    def test_article_settings_move_nav(self):
        self.test_article_settings(True)

    def test_article_settings_on_two_sites(self):
        other_site = mommy.make(Site)

        initial_data = {'title': "test", 'content': "this is my article content"}
        article_class = get_article_class()

        art1 = article_class.objects.create(publication=BaseArticle.PUBLISHED, **initial_data)

        self._log_as_editor()
        data = {
            'template': get_article_templates(None, self.user)[0][0],
            'category': '',
            'publication': BaseArticle.PUBLISHED,
            'publication_date': "2013-01-01 12:00:00",
            'headline': True,
            'in_newsletter': True,
            'summary': 'short summary',
            'navigation_parent': None,
            'sites': [settings.SITE_ID, other_site.id]
        }

        response = self.client.post(reverse('coop_cms_article_settings', args=[art1.id]), data=data, follow=True)
        self.assertEqual(response.status_code, 200)

        self.assertEqual(article_class.objects.exclude(id=art1.id).count(), 0)
        art1 = article_class.objects.get(id=art1.id)

        self.assertEqual(art1.summary, data['summary'])
        self.assertEqual(sorted([a.id for a in art1.sites.all()]), sorted(data['sites']))

    def test_article_settings_unknown_sites(self):
        other_site = mommy.make(Site)

        initial_data = {'title': "test", 'content': "this is my article content"}
        article_class = get_article_class()

        art1 = article_class.objects.create(publication=BaseArticle.PUBLISHED, **initial_data)

        self._log_as_editor()
        data = {
            'template': get_article_templates(None, self.user)[0][0],
            'category': '',
            'publication': BaseArticle.PUBLISHED,
            'publication_date': "2013-01-01 12:00:00",
            'headline': True,
            'in_newsletter': True,
            'summary': 'short summary',
            'navigation_parent': None,
            'sites': [settings.SITE_ID, 999]
        }

        response = self.client.post(reverse('coop_cms_article_settings', args=[art1.id]), data=data, follow=True)
        self.assertEqual(response.status_code, 200)

        soup = BeautifulSoup(response.content)
        self.assertEqual(len(soup.select("ul.errorlist")), 1)

        self.assertEqual(article_class.objects.exclude(id=art1.id).count(), 0)
        art1 = article_class.objects.get(id=art1.id)

        self.assertNotEqual(art1.summary, data['summary'])
        self.assertEqual(sorted([a.id for a in art1.sites.all()]), [settings.SITE_ID])

    def test_article_settings_no_sites(self):
        other_site = mommy.make(Site)

        initial_data = {'title': "test", 'content': "this is my article content"}
        article_class = get_article_class()

        art1 = article_class.objects.create(publication=BaseArticle.PUBLISHED, **initial_data)

        self._log_as_editor()
        data = {
            'template': get_article_templates(None, self.user)[0][0],
            'category': '',
            'publication': BaseArticle.PUBLISHED,
            'publication_date': "2013-01-01 12:00:00",
            'headline': True,
            'in_newsletter': True,
            'summary': 'short summary',
            'navigation_parent': None,
            'sites': []
        }

        response = self.client.post(reverse('coop_cms_article_settings', args=[art1.id]), data=data, follow=True)
        self.assertEqual(response.status_code, 200)

        soup = BeautifulSoup(response.content)
        self.assertEqual(len(soup.select("ul.errorlist")), 1)

        self.assertEqual(article_class.objects.exclude(id=art1.id).count(), 0)
        art1 = article_class.objects.get(id=art1.id)

        self.assertNotEqual(art1.summary, data['summary'])
        self.assertEqual(sorted([a.id for a in art1.sites.all()]), [settings.SITE_ID])



# -*- coding: utf-8 -*-
"""
Coop_cms settings : central place for coop_cms settings
the settings should be accessed from here and not directly from django.conf.settings
"""

import os.path
import sys

from django.conf import settings as django_settings
from django.conf.urls import patterns
from django.conf.urls.i18n import i18n_patterns
from django.contrib.contenttypes.models import ContentType
from django.contrib.sites.models import Site
from importlib import import_module

from coop_cms.logger import logger


DEFAULT_NAVTREE_CLASS = COOP_CMS_NAVTREE_CLASS = 'coop_cms.NavTree'
DEPRECATED_COOP_CMS_NAVTREE_CLASS = getattr(django_settings, 'COOP_CMS_NAVTREE_CLASS', 'basic_cms.NavTree')
DEFAULT_MEDIA_ROOT = ''


def load_class(settings_key, default_value):
    """returns the form to be used for creating a new article"""
    full_class_name = getattr(django_settings, settings_key, '') or default_value
    if full_class_name:
        try:
            module_name, class_name = full_class_name.rsplit('.', 1)
        except ValueError:
            raise ImportError("Unable to import {0}: full path is required".format(full_class_name))
        module = import_module(module_name)
        class_object = getattr(module, class_name)
        return class_object
    return None


def get_navigable_content_types():
    """returns the list of content types of navigable (which can be used in navigation) models"""
    ct_choices = []
    try:
        content_apps = django_settings.COOP_CMS_CONTENT_APPS
    except AttributeError:
        content_apps = []
        not_to_be_mapped = ('south', 'django_extensions', 'd2rq')
        for module in django_settings.INSTALLED_APPS:
            if (not module.startswith('django.')) and (module not in not_to_be_mapped):
                content_apps.append(module)
    apps_labels = [app.rsplit('.')[-1] for app in content_apps]
    navigable_content_types = ContentType.objects.filter(app_label__in=apps_labels).order_by('app_label')
    for content_type in navigable_content_types:
        is_navnode = ((content_type.model == 'navnode') and (content_type.app_label == 'coop_cms'))
        if (not is_navnode) and 'get_absolute_url' in dir(content_type.model_class()):
            ct_choices.append((content_type.id, content_type.app_label + u'.' + content_type.model))
    return ct_choices


def get_navtree_class(defaut_class=None):
    """
    returns the custom navtree class
    Warning : It is not recommend to define a custom NavTree. This feature is deprecated
    """
    if hasattr(get_navtree_class, '_cache_class'):
        return getattr(get_navtree_class, '_cache_class')
    else:
        navtree_class = None
        if DEFAULT_NAVTREE_CLASS != COOP_CMS_NAVTREE_CLASS:
            full_class_name = COOP_CMS_NAVTREE_CLASS
            app_label, model_name = full_class_name.split('.')
            model_name = model_name.lower()
            try:
                content_type = ContentType.objects.get(app_label=app_label, model=model_name)
                navtree_class = content_type.model_class()
            except Exception:
                navtree_class = None

        if navtree_class is None:
            module = import_module('coop_cms.models')
            navtree_class = module.NavTree
        setattr(get_navtree_class, '_cache_class', navtree_class)
        return navtree_class


def get_article_class():
    """
    returns the custom Article class
    This makes possible to customize the Article model. However, It must inherit from BaseArticle
    """
    if hasattr(get_article_class, '_cache_class'):
        return getattr(get_article_class, '_cache_class')
    else:
        default_value = ""
        if 'coop_cms.apps.basic_cms' in django_settings.INSTALLED_APPS:
            default_value = 'coop_cms.apps.basic_cms.models.Article'

        article_class = load_class('COOP_CMS_ARTICLE_CLASS', default_value)
        if not article_class:
            raise Exception('No article class configured')

        setattr(get_article_class, '_cache_class', article_class)
        return article_class


def get_default_logo():
    """returns the default logo"""
    return getattr(django_settings, 'COOP_CMS_DEFAULT_ARTICLE_LOGO', 'img/default-logo.png')


def get_article_form():
    """returns a form to be used for editing an article"""
    return load_class('COOP_CMS_ARTICLE_FORM', 'coop_cms.forms.ArticleForm')


def get_article_settings_form():
    """returns the form to use for editing article settings"""
    return load_class('COOP_CMS_ARTICLE_SETTINGS_FORM', 'coop_cms.forms.ArticleSettingsForm')


def get_new_article_form():
    """returns the form to be used for creating a new article"""
    return load_class('COOP_CMS_NEW_ARTICLE_FORM', 'coop_cms.forms.NewArticleForm')


def get_newsletter_templates(newsletter, user):
    """returns the list of newsletter templates"""
    try:
        return getattr(django_settings, 'COOP_CMS_NEWSLETTER_TEMPLATES')
    except AttributeError:
        return ()


def get_newsletter_form():
    """returns the form to use for editing a newsletter"""
    return load_class('COOP_CMS_NEWSLETTER_FORM', 'coop_cms.forms.NewsletterForm')


def get_newsletter_settings_form():
    """returns the form to use for for newsletter settings"""
    return load_class('COOP_CMS_NEWSLETTER_SETTINGS_FORM', 'coop_cms.forms.NewsletterSettingsForm')


def get_article_templates(article, user):
    """returns the list of article templates"""
    if hasattr(django_settings, 'COOP_CMS_ARTICLE_TEMPLATES'):
        coop_cms_article_templates = getattr(django_settings, 'COOP_CMS_ARTICLE_TEMPLATES')

        if type(coop_cms_article_templates) in (str, unicode):
            # COOP_CMS_ARTICLE_TEMPLATES is a string :
            #  - a function name that will return a tuple
            #  - a variable name that contains a tuple

            # extract module and function/var names
            module_name, object_name = coop_cms_article_templates.rsplit('.', 1)
            module = import_module(module_name) # import module
            article_templates_object = getattr(module, object_name)  # get the object
            if callable(article_templates_object):
                # function: call it
                article_templates = article_templates_object(article, user)
            else:
                # var: assign
                article_templates = article_templates_object
        else:
            # COOP_CMS_ARTICLE_TEMPLATES is directly a tuple, assign it
            article_templates = coop_cms_article_templates
    else:
        article_templates = None

    return article_templates


def _get_article_setting(article, setting_name, default_value):
    """private function: access an article-dependant setting"""
    try:
        get_setting_name = getattr(django_settings, setting_name)
        try:
            module_name, fct_name = get_setting_name.rsplit('.', 1)
            module = import_module(module_name)
            get_setting = getattr(module, fct_name)
            if callable(get_setting):
                #If the setting is a function get the value as return value of the function call
                value = get_setting(article)
            else:
                #else Take the value as it is
                value = get_setting
        except ValueError:
            value = get_setting_name

    except AttributeError:
        value = default_value
    return value


def get_article_logo_size(article):
    """get the article logo size"""
    return _get_article_setting(article, 'COOP_CMS_ARTICLE_LOGO_SIZE', '48x48')


def get_article_logo_crop(article):
    """get the article logo crop"""
    return _get_article_setting(article, 'COOP_CMS_ARTICLE_LOGO_CROP', 'center')


def get_headline_image_size(article):
    """get the headline image size"""
    return _get_article_setting(article, 'COOP_CMS_HEADLINE_IMAGE_SIZE', '900')


def get_headline_image_crop(article):
    """get the headline image crop"""
    return _get_article_setting(article, 'COOP_CMS_HEADLINE_IMAGE_CROP', None)


def get_max_image_width(image):
    """get the ax image width: avoid user to use very large image"""
    return _get_article_setting(image, 'COOP_CMS_MAX_IMAGE_WIDTH', None)


def get_newsletter_item_classes():
    """get items thant can be used in newsletter"""
    if hasattr(get_newsletter_item_classes, '_cache_class'):
        return getattr(get_newsletter_item_classes, '_cache_class')
    else:
        item_classes = []
        try:
            full_classes_names = getattr(django_settings, 'COOP_CMS_NEWSLETTER_ITEM_CLASSES')
        except AttributeError:
            item_classes = (get_article_class(),)
        else:
            item_classes = []
            for full_class_name in full_classes_names:
                module_name, class_name = full_class_name.rsplit('.', 1)
                module = import_module(module_name)
                item_classes.append(getattr(module, class_name))
            item_classes = tuple(item_classes)

        if not item_classes:
            raise Exception('No newsletter item classes configured')

        setattr(get_newsletter_item_classes, '_cache_class', item_classes)
        return item_classes


def get_newsletter_context_callbacks():
    """get the context for newsletter template"""
    if hasattr(get_newsletter_context_callbacks, '_cache_func'):
        return getattr(get_newsletter_context_callbacks, '_cache_func')
    else:
        try:
            callback_names = getattr(django_settings, 'COOP_CMS_NEWSLETTER_CONTEXT')
        except AttributeError:
            return ()
        else:
            callbacks = []
            for callback_name in callback_names:
                module_name, func_name = callback_name.rsplit('.', 1)
                module = import_module(module_name)
                callbacks.append(getattr(module, func_name))
            callbacks = tuple(callbacks)

        setattr(get_newsletter_context_callbacks, '_cache_func', callbacks)
        return callbacks


def is_localized():
    """return True if site is localized"""
    if ('modeltranslation' in django_settings.INSTALLED_APPS):
        return True
    return False


def is_multilang():
    """return true if several languages are set"""
    return len(django_settings.LANGUAGES) > 1


def multilang_mode():
    """return true if several languages are set"""
    return len(django_settings.LANGUAGES)


def install_csrf_failure_view():
    """Make possible to customize csrf failure page"""
    dont_do_it = getattr(django_settings, 'COOP_CMS_DO_NOT_INSTALL_CSRF_FAILURE_VIEW', False)
    if not dont_do_it:
        setattr(django_settings, 'CSRF_FAILURE_VIEW', 'coop_cms.views.webutils.csrf_failure')


def cms_no_homepage():
    """returns true if homepage is not managed by coop_cms"""
    return getattr(django_settings, 'COOP_CMS_NO_HOMEPAGE', False)


def hide_media_library_menu():
    """returns True if media is not displayed in coop_bar menu"""
    return getattr(django_settings, 'COOP_CMS_HIDE_MEDIA_LIBRARY_MENU', False)


def is_requestprovider_installed():
    """returns True if possible to get request from anywhere in the code"""
    is_installed = ('coop_cms.utils.RequestMiddleware' in django_settings.MIDDLEWARE_CLASSES)
    if not is_installed:
        logger.warn("You should add coop_cms.utils.RequestMiddleware to the MIDDLEWARE_CLASSES settings")
    return is_installed


def can_rewrite_url():
    """returns True if user is allowed to change article slugs"""
    return getattr(django_settings, 'COOP_CMS_CAN_EDIT_ARTICLE_SLUG', False)


def get_article_views():
    """returns article views"""
    try:
        article_views = getattr(django_settings, 'COOP_CMS_ARTICLE_VIEWS')
        return article_views
    except AttributeError:
        from coop_cms.views.articles import ArticleView
        return {
            'article_view': ArticleView,
            'edit_article_view': ArticleView,
        }


def is_perm_middleware_installed():
    """returns True if permission middleware is installed"""
    return 'coop_cms.middleware.PermissionsMiddleware' in django_settings.MIDDLEWARE_CLASSES


# Check that languages are correctly set
if is_localized():
    if django_settings.LANGUAGE_CODE[:2] != django_settings.LANGUAGES[0][0]:
        text = "coop_cms settings error: LANGUAGE_CODE ({0}) should be first in LANGUAGES (currently first is {1})"
        text = text.format(django_settings.LANGUAGE_CODE[:2], django_settings.LANGUAGES[0][0])
        logger.warning(text)


def is_multi_site():
    """returns True if several sites are configured"""
    return Site.objects.count() > 1


def get_img_folder(instance, filename):
    """image folder"""
    try:
        img_root = django_settings.IMAGE_FOLDER
    except AttributeError:
        img_root = 'img'

    return u'{0}/{1}'.format(img_root, filename)


def get_articles_category_page_size(article_category):
    """returns number of articles for pagination"""
    if article_category.pagination_size:
        return article_category.pagination_size
    return getattr(django_settings, 'COOP_CMS_ARTICLES_CATEGORY_PAGINATION', 10)


def get_url_patterns():
    """return urlspatterns to use"""
    if is_localized():
        return i18n_patterns
    else:
        def url_list(*args):
            return list(args)
        return url_list


def get_unit_test_media_root():
    """return unit testing_media root"""
    global DEFAULT_MEDIA_ROOT
    if not DEFAULT_MEDIA_ROOT:
        DEFAULT_MEDIA_ROOT = django_settings.MEDIA_ROOT
        django_settings.MEDIA_ROOT = os.path.join(django_settings.MEDIA_ROOT, '_unit_tests')
    return django_settings.MEDIA_ROOT


def get_media_root():
    """return unit testing_media root if unit test, regular unit test if not"""
    if 'test' in sys.argv:
        return get_unit_test_media_root()
    else:
        return django_settings.MEDIA_ROOT



# -*- coding: utf-8 -*-
"""modeltranslation settings"""

from modeltranslation.translator import translator, TranslationOptions # pylint: disable=F0401

from coop_cms.models import Alias, ArticleCategory, Fragment, NavNode, Newsletter, PieceOfHtml, SiteSettings


class PieceOfHtmlTranslationOptions(TranslationOptions):
    """translation"""
    fields = ('content',)
translator.register(PieceOfHtml, PieceOfHtmlTranslationOptions)


class FragmentTranslationOptions(TranslationOptions):
    """translation"""
    fields = ('content',)
translator.register(Fragment, FragmentTranslationOptions)


class NavNodeTranslationOptions(TranslationOptions):
    """translation"""
    fields = ('label',)
translator.register(NavNode, NavNodeTranslationOptions)


class ArticleCategoryTranslationOptions(TranslationOptions):
    """translation"""
    fields = ('name',)
translator.register(ArticleCategory, ArticleCategoryTranslationOptions)


class SiteSettingsTranslationOptions(TranslationOptions):
    """translation"""
    fields = ('homepage_url',)
translator.register(SiteSettings, SiteSettingsTranslationOptions)


class AliasTranslationOptions(TranslationOptions):
    """translation"""
    fields = ('path', 'redirect_url',)
translator.register(Alias, AliasTranslationOptions)


class NewsletterTranslationOptions(TranslationOptions):
    """translation"""
    fields = ('subject', 'content',)
translator.register(Newsletter, NewsletterTranslationOptions)



# -*- coding:utf-8 -*-
"""article tags"""

from django import template

register = template.Library()

from coop_cms.settings import get_article_class


@register.tag
def last_articles(parser, token):
    """get last articles"""
    try:
        args = token.split_contents()
        if len(args) == 4:
            tag_name, number, template, category = token.split_contents()
            return ArticleListNode(number, template, category)
        elif len(args) == 3:
            tag_name, number, template = token.split_contents()
            return ArticleListNode(number, template, None)
    except ValueError:
        raise template.TemplateSyntaxError('%s tag requires at least 2 arguments' % token.split_contents()[0])


def resolve(var_or_value, ctx):
    """resolve context"""
    if var_or_value[0] == '"':
        return var_or_value[1:-1]
    return ctx.resolve_variable(var_or_value)


class ArticleListNode(template.Node):
    """
    This templatetag takes as arguments :
    - A number limit of articles #TODO pagination
    - A template to render each articles
    - An optional ArticleCategory object
    """

    def __init__(self, number, the_template, category):
        self.number = number
        self.the_template = the_template
        if category:
            self.category = template.Variable(category)

    def last_articles(self, context):
        """return latest articles"""
        article_list = []
        all_articles = get_article_class().objects.all().order_by('-created')
        if hasattr(self, 'category'):
            category = self.category.resolve(context)
            all_articles = all_articles.filter(category=category)
        all_articles = all_articles[:self.number]
        for article in all_articles:
            if article.navigation_parent is None:
                article_list.append(article)
        return article_list

    def render(self, context):
        """convert to html"""
        template_name = resolve(self.the_template, context)
        the_template = template.loader.get_template(template_name)
        return ''.join([the_template.render(template.Context({'item': item})) for item in self.last_articles(context)])
